<?
class SiteMap {
	
	/**
	 * @desc
	 * Array con urls.
	 *
	 * @var array
	 */
	private $Urls;
	
	/**
	 * @desc
	 * Buffer temporal.
	 *
	 * @var string
	 */
	public $Buffer = '';
	
	/**
	 * @desc
	 * Codificacion de salida.
	 *
	 * @var unknown_type
	 */
	public $Encoding = 'UTF-8';
	
	function __construct() {
		
	}
	
	/**
	 * @desc
	 * Genera el codigo del SiteMap y lo almacena en el buffer.
	 * Si se indica $nShow en verdadero envia la salida a pantalla.
	 *
	 * @param bool $nShow
	 */
	public function Generate($nShow = true) {
		// Encabezados
		$this->Buffer = "<?xml version=\"1.0\" encoding=\"{$this->Encoding}\"?>";
		$this->Buffer .= "\n";
		$this->Buffer .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"';
		$this->Buffer .= "\n\t";
		$this->Buffer .= 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"';
		$this->Buffer .= "\n\t";
		$this->Buffer .= 'xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9';
		$this->Buffer .= "\n\t";
		$this->Buffer .= 'http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd">';
		
		// SiteMap
		foreach ($this->Urls as $aItem) {
			$this->Buffer .= "\t<url>\n";
			$this->Buffer .= "\t\t<loc>".$aItem['loc']."</loc>\n";
			$this->Buffer .= "\t\t<lastmod>".$aItem['lastmod']."</lastmod>\n";
			$this->Buffer .= "\t\t<changefreq>".$aItem['changefreq']."</changefreq>\n";
			$this->Buffer .= "\t\t<priority>".$aItem['priority']."</priority>\n";
			$this->Buffer .= "\t</url>\n";
		}
		
		$this->Buffer .= "</urlset>";
		
		if ($nShow) {
			header("Content-type: application/xhtml+xml\n");
			echo $this->Buffer;
		}
	}
	
	/**
	 * @desc
	 * Guarda el buffer en la ubicacion indicada.
	 *
	 * @param string $cFileName
	 */
	public function Save($cFileName = 'sitemap.xml') {
		$sFp = fopen($cFileName, 'w');
		fwrite($sFp, $this->Buffer);
		fclose($sFp);
	}
	
	/**
	 * @desc
	 * Agrega una URL al sitemap.
	 * 
	 * <b>$cUrl</b>: URL. Las entidades HTML son convertidas automaticamente.
	 * <b>$nLastMod</b>: Fecha de ultima modificacion, por defecto vale una marca de tiempo actual.
	 * <b>$cChangeFreq</b>: Frecuencia de actualizacion del url (daily).
	 * <b>$nPriority</b>: Prioridad (0 a 1).
	 *
	 * @param string $cUrl
	 * @param float $nPriority
	 * @param string $cChangeFreq
	 * @param int $nLastMod
	 */
	public function AddUrl($cUrl, $nPriority=0.5, $cChangeFreq='daily', $nLastMod=null) {
		// URL
		$cUrl =  htmlentities($cUrl, ENT_QUOTES, $this->Encoding);
		
		// LastMod
		if ($nLastMod === null) {
			$nLastMod = time();
		}
		$cLastMod = date('c', $nLastMod);
			
		// ChangeFreq
		if ($cChangeFreq === null) {
			$cChangeFreq = 'daily';
		}
		
		// Prioriry
		if ($nPriority < 0 || $nPriority > 1) {
			$nPriority = 0.5;
		}
			
		$this->Urls[] = array('loc'=>$cUrl, 'lastmod'=>$cLastMod, 'changefreq'=>$cChangeFreq, 'priority'=>$nPriority);
	}
}
?>