<?
class Alertas_operador extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Cadena con el GROUP BY.
	 *
	 * @var string
	 */
	public $GroupBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdAlerta, $IdOrigen, $IdDestino, $Mensaje, $Visto;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Alertas_operador.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cGroupBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Agrupacion.
		if ($this->GroupBy) {
			$cGroupBy = "GROUP BY {$this->GroupBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdOrigen !== null) {
			$aWhere[] = "alertas_operador.id_origen = '{$this->IdOrigen}'";
		}
		if ($this->IdDestino !== null) {
			$aWhere[] = "alertas_operador.id_destino = '{$this->IdDestino}'";
		}
		if ($this->Mensaje !== null) {
			$aWhere[] = "alertas_operador.mensaje = '{$this->Mensaje}'";
		}
		if ($this->Visto !== null) {
			$aWhere[] = "alertas_operador.visto = '{$this->Visto}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS 
			alertas_operador.*, operadores.nombre origen
			FROM alertas_operador 
			INNER JOIN elementos ON elementos.id_entidad = alertas_operador.id_alerta AND elementos.id_tipoelemento = ".ELEMENTOS_ALERTAOPERADOR." AND elementos.eliminado = 0 
			LEFT JOIN operadores ON operadores.id_operador = alertas_operador.id_origen
			{$cWhere} {$cGroupBy} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Alertas_operador con ID $this->IdAlerta.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT alertas_operador.* FROM alertas_operador WHERE alertas_operador.id_alerta = '{$this->IdAlerta}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Alertas_operador.
	 * Devuelve el ID del Alertas_operador guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdAlerta) {
			$cSql = "UPDATE alertas_operador SET ".
				"id_origen = '{$this->IdOrigen}', ".
				"id_destino = '{$this->IdDestino}', ".
				"mensaje = ".($this->Mensaje ? "'{$this->Mensaje}'" : 'NULL').", ".
				"visto = '{$this->Visto}' ".
				"WHERE id_alerta = '{$this->IdAlerta}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO alertas_operador SET ".
				"id_origen = '{$this->IdOrigen}', ".
				"id_destino = '{$this->IdDestino}', ".
				"mensaje = ".($this->Mensaje ? "'{$this->Mensaje}'" : 'NULL').", ".
				"visto = '{$this->Visto}'";
			
			$this->IdAlerta = $this->DB->QueryInsert($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_ALERTAOPERADOR, $this->IdAlerta);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdAlerta;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Alertas_operador.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		Utils::EliminarElemento(ELEMENTOS_ALERTAOPERADOR, $this->IdAlerta);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdAlerta = $aDatos['id_alerta'];
		$this->IdOrigen = $aDatos['id_origen'];
		$this->IdDestino = $aDatos['id_destino'];
		$this->Mensaje = $aDatos['mensaje'];
		$this->Visto = $aDatos['visto'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdAlerta = $this->IdOrigen = $this->IdDestino = $this->Mensaje = $this->Visto = null;
	}
}
?>