<?
class Bancos extends Model {
	
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdBanco, $Codigo, $Nombre;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Bancos.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Codigo !== null) {
			$aWhere[] = "bancos.codigo = '{$this->Codigo}'";
		}
		if ($this->Nombre !== null) {
			$aWhere[] = "bancos.nombre LIKE '%{$this->Nombre}%'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "bancos.nombre REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "bancos.nombre LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS bancos.* FROM bancos {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Bancos con ID $this->IdBanco.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT bancos.* FROM bancos WHERE bancos.id_banco = '{$this->IdBanco}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Bancos.
	 * Devuelve el ID del Bancos guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		if ($this->IdBanco) {
			$cSql = "UPDATE bancos SET ".
				"codigo = '{$this->Codigo}', ".
				"nombre = '{$this->Nombre}' ".
				"WHERE id_banco = '{$this->IdBanco}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO bancos SET ".
				"codigo = '{$this->Codigo}', ".
				"nombre = '{$this->Nombre}'";
			
			$this->IdBanco = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			return 0;
		}
		else {
			return $this->IdBanco;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Bancos.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE FROM bancos WHERE id_banco = '{$this->IdBanco}' LIMIT 1";
		$this->DB->Query($cSql);
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdBanco = $aDatos['id_banco'];
		$this->Codigo = $aDatos['codigo'];
		$this->Nombre = $aDatos['nombre'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdBanco = $this->Codigo = $this->Nombre = null;
	}
}
?>