<?
class Cheques extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdCheque, $IdCliente, $IdBanco, $Numero, $FechaIngreso, $FechaVencimiento, $Importe, $Depositado, $Cobrado, $IdProveedor, $FechaEgreso;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Cheques.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = $aWhereOr = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdCliente !== null) {
			$aWhere[] = "cheques.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdBanco !== null) {
			$aWhere[] = "cheques.id_banco = '{$this->IdBanco}'";
		}
		if ($this->Numero !== null) {
			$aWhere[] = "cheques.numero = '{$this->Numero}'";
		}
		if ($this->FechaIngreso !== null) {
			$aWhere[] = "cheques.fecha_ingreso = '{$this->FechaIngreso}'";
		}
		if ($this->FechaVencimiento !== null) {
			$aWhere[] = "cheques.fecha_vencimiento = '{$this->FechaVencimiento}'";
		}
		if ($this->Importe !== null) {
			$aWhere[] = "cheques.importe = '{$this->Importe}'";
		}
		if ($this->Depositado !== null) {
			$aWhere[] = "cheques.depositado = '{$this->Depositado}'";
		}
		if ($this->Cobrado !== null) {
			$aWhere[] = "cheques.cobrado = '{$this->Cobrado}'";
		}
		if ($this->IdProveedor !== null) {
			$aWhere[] = "cheques.id_proveedor = '{$this->IdProveedor}'";
		}
		if ($this->FechaEgreso !== null) {
			$aWhere[] = "cheques.fecha_egreso = '{$this->FechaEgreso}'";
		}
		if (@$this->FueEntregado === true) {
			$aWhereOr[] = "cheques.id_proveedor IS NOT NULL";
		}
		if (@$this->FueEntregado === false) {
			$aWhere[] = "cheques.id_proveedor IS NULL";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		$cWhereOr = $aWhereOr ? join(' OR ', $aWhereOr) : '';
		
		if ($cWhere && $cWhereOr) {
			$cWhere .= " OR ({$cWhereOr})";
		}
		elseif(!$cWhere && $cWhereOr) {
			$cWhere = $cWhereOr;
		}
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS cheques.*, clientes.nombre cliente,bancos.nombre banco
			FROM cheques 
			INNER JOIN elementos ON elementos.id_entidad = cheques.id_cheque AND elementos.id_tipoelemento = ".ELEMENTOS_CHEQUE." AND elementos.eliminado = 0
			LEFT JOIN clientes ON clientes.id_cliente = cheques.id_cliente
			LEFT JOIN bancos ON bancos.id_banco = cheques.id_banco
			{$cWhere} {$cOrderBy} {$cLimit}";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Cheques con ID $this->IdCheque.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT cheques.*, clientes.nombre cliente, bancos.nombre banco, proveedores.denominacion proveedor
			FROM cheques 
			LEFT JOIN clientes ON clientes.id_cliente = cheques.id_cliente
			LEFT JOIN bancos ON bancos.id_banco = cheques.id_banco
			LEFT JOIN proveedores ON proveedores.id_proveedor = cheques.id_proveedor
			WHERE cheques.id_cheque = '{$this->IdCheque}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Cheques.
	 * Devuelve el ID del Cheques guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdCheque) {
			$cSql = "UPDATE cheques SET ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_banco = ".($this->IdBanco ? "'{$this->IdBanco}'" : 'NULL').", ".
				"numero = ".($this->Numero ? "'{$this->Numero}'" : 'NULL').", ".
				"fecha_ingreso = '{$this->FechaIngreso}', ".
				"fecha_vencimiento = '{$this->FechaVencimiento}', ".
				"importe = '{$this->Importe}', ".
				"depositado = '{$this->Depositado}', ".
				"cobrado = '{$this->Cobrado}', ".
				"id_proveedor = ".($this->IdProveedor ? "'{$this->IdProveedor}'" : 'NULL').", ".
				"fecha_egreso = ".($this->FechaEgreso ? "'{$this->FechaEgreso}'" : 'NULL')." ".
				"WHERE id_cheque = '{$this->IdCheque}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO cheques SET ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_banco = ".($this->IdBanco ? "'{$this->IdBanco}'" : 'NULL').", ".
				"numero = ".($this->Numero ? "'{$this->Numero}'" : 'NULL').", ".
				"fecha_ingreso = '{$this->FechaIngreso}', ".
				"fecha_vencimiento = '{$this->FechaVencimiento}', ".
				"importe = '{$this->Importe}', ".
				"depositado = '{$this->Depositado}', ".
				"cobrado = '{$this->Cobrado}', ".
				"id_proveedor = ".($this->IdProveedor ? "'{$this->IdProveedor}'" : 'NULL').", ".
				"fecha_egreso = ".($this->FechaEgreso ? "'{$this->FechaEgreso}'" : 'NULL');
			
			$this->IdCheque = $this->DB->QueryInsert($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_CHEQUE, $this->IdCheque);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdCheque;
		}
	}
	
	/**
	 * @desc
	 * Devuelve un array con la cantidad de cheques y saldo por cobrar de cheques vencidos.
	 *
	 * @return array
	 */
	public function SaldoPorCobrar($nReVencidos=false) {
		
		$cWhereFecha = $nReVencidos ? 'DATE_SUB(CURRENT_DATE(), INTERVAL 25 DAY)' : 'CURRENT_DATE()';
		
		$cSql = "SELECT COUNT(cheques.id_cheque) cantidad, SUM(importe) importe
		FROM cheques
		INNER JOIN elementos ON elementos.id_entidad = cheques.id_cheque AND elementos.id_tipoelemento = ".ELEMENTOS_CHEQUE." AND elementos.eliminado = 0
		WHERE cheques.depositado = 0 AND cheques.fecha_vencimiento <= {$cWhereFecha}
		";
		
		$this->DB->Query($cSql);
		
		return $this->DB->Field();
	}
	
	/**
	 * @desc
	 * Elimina un Cheques.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		Utils::EliminarElemento(ELEMENTOS_CHEQUE, $this->IdCheque);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdCheque = $aDatos['id_cheque'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->IdBanco = $aDatos['id_banco'];
		$this->Numero = $aDatos['numero'];
		$this->FechaIngreso = $aDatos['fecha_ingreso'];
		$this->FechaVencimiento = $aDatos['fecha_vencimiento'];
		$this->Importe = $aDatos['importe'];
		$this->Depositado = $aDatos['depositado'];
		$this->Cobrado = $aDatos['cobrado'];
		$this->IdProveedor = $aDatos['id_proveedor'];
		$this->FechaEgreso = $aDatos['fecha_egreso'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdCheque = $this->IdCliente = $this->IdBanco = $this->Numero = $this->FechaIngreso = $this->FechaVencimiento = $this->Importe = $this->Depositado = $this->Cobrado = $this->IdProveedor = $this->FechaEgreso = null;
	}
}
?>