<?
class Clientes extends Model {
	
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdCliente, $Nombre, $Categoria, $RazonSocial, $IdCondicionfiscal, $Cuit, $Domicilio, $DomicilioComercial, $DomicilioCorrespondencia, $Telefono, $PaginaWeb, $Email, $IdComercial, $Bloqueado, $Comentario, $Fecha_alta;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Clientes.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Nombre !== null) {
			$aWhere[] = "clientes.nombre LIKE '{$this->Nombre}%'";
		}
		if ($this->Categoria !== null) {
			$aWhere[] = "clientes.categoria = '{$this->Categoria}'";
		}
		if (strtolower($this->Categoria) == "z") {
			$aWhere[] = "clientes.categoria <> 'z' OR clientes.categoria <> 'Z'";
		}
		if ($this->RazonSocial !== null) {
			$aWhere[] = "clientes.razon_social LIKE '%{$this->RazonSocial}%'";
		}
		if ($this->IdCondicionfiscal !== null) {
			$aWhere[] = "clientes.id_condicionfiscal = '{$this->IdCondicionfiscal}'";
		}
		if ($this->Cuit !== null) {
			$aWhere[] = "clientes.cuit = '{$this->Cuit}'";
		}
		if ($this->Domicilio !== null) {
			$aWhere[] = "clientes.domicilio = '{$this->Domicilio}'";
		}
		if ($this->Telefono !== null) {
			$aWhere[] = "clientes.telefono = '{$this->Telefono}'";
		}
		if ($this->PaginaWeb !== null) {
			$aWhere[] = "clientes.pagina_web = '{$this->PaginaWeb}'";
		}
		if ($this->Email !== null) {
			$aWhere[] = "clientes.email = '{$this->Email}'";
		}
		if ($this->IdComercial !== null) {
			$aWhere[] = "clientes.id_comercial = '{$this->IdComercial}'";
		}
		if ($this->Bloqueado !== null) {
			$aWhere[] = "clientes.bloqueado = '{$this->Bloqueado}'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "clientes.nombre REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "clientes.nombre LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS clientes.*, condiciones_fiscales.nombre condicion_fiscal, condiciones_fiscales.letra comprobante, condiciones_fiscales.iva discrimina_iva
			FROM clientes 
			LEFT JOIN condiciones_fiscales ON condiciones_fiscales.id_condicionfiscal = clientes.id_condicionfiscal
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	public function Contador() {
		
		$cSql = "SELECT count(*) as cantidad FROM clientes";
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		return $aRet['cantidad'];
	
	}
	
	public function UltimoCliente() {
		
		$cSql = "SELECT * FROM clientes ORDER BY id_cliente DESC LIMIT 1";
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		return $aRet;
	
	}
	
	public function ContadorProyectos() {
		
		$cSql = "SELECT count(*) as cantidad FROM proyectos";
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		return $aRet['cantidad'];
	
	}
	
	/**
	 * @desc
	 * Devuelve el Clientes con ID $this->IdCliente.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT clientes.*, operadores.nombre comercial, condiciones_fiscales.nombre condicion_fiscal, condiciones_fiscales.grava
			FROM clientes 
			LEFT JOIN operadores ON clientes.id_comercial = operadores.id_operador
			LEFT JOIN condiciones_fiscales ON condiciones_fiscales.id_condicionfiscal = clientes.id_condicionfiscal
			WHERE clientes.id_cliente = '{$this->IdCliente}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Clientes.
	 * Devuelve el ID del Clientes guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdCliente) {
			$cSql = "UPDATE clientes SET ".
				"nombre = '{$this->Nombre}', ".
				"categoria = '{$this->Categoria}', ".
				"razon_social = ".($this->RazonSocial ? "'{$this->RazonSocial}'" : 'NULL').", ".
				"id_condicionfiscal = ".($this->IdCondicionfiscal ? "'{$this->IdCondicionfiscal}'" : 'NULL').", ".
				"cuit = ".($this->Cuit ? "'{$this->Cuit}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"domicilio_comercial = ".($this->DomicilioComercial ? "'{$this->DomicilioComercial}'" : 'NULL').", ".
				"domicilio_correspondencia = ".($this->DomicilioCorrespondencia ? "'{$this->DomicilioCorrespondencia}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"pagina_web = ".($this->PaginaWeb ? "'{$this->PaginaWeb}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"id_comercial = ".($this->IdComercial ? "'{$this->IdComercial}'" : 'NULL').", ".
				"bloqueado = '{$this->Bloqueado}', ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL')." ".
				"WHERE id_cliente = '{$this->IdCliente}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO clientes SET ".
				"nombre = '{$this->Nombre}', ".
				"categoria = '{$this->Categoria}', ".
				"razon_social = ".($this->RazonSocial ? "'{$this->RazonSocial}'" : 'NULL').", ".
				"id_condicionfiscal = ".($this->IdCondicionfiscal ? "'{$this->IdCondicionfiscal}'" : 'NULL').", ".
				"cuit = ".($this->Cuit ? "'{$this->Cuit}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"domicilio_comercial = ".($this->DomicilioComercial ? "'{$this->DomicilioComercial}'" : 'NULL').", ".
				"domicilio_correspondencia = ".($this->DomicilioCorrespondencia ? "'{$this->DomicilioCorrespondencia}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"pagina_web = ".($this->PaginaWeb ? "'{$this->PaginaWeb}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"id_comercial = ".($this->IdComercial ? "'{$this->IdComercial}'" : 'NULL').", ".
				"bloqueado = '{$this->Bloqueado}', ".
				"fecha_alta = '{$this->Fecha_alta}', ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL')."";
			
			$this->IdCliente = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdCliente;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Clientes.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE FROM clientes WHERE id_cliente = '{$this->IdCliente}' LIMIT 1";
		$this->DB->Query($cSql);
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdCliente = $aDatos['id_cliente'];
		$this->Nombre = $aDatos['nombre'];
		$this->Categoria = $aDatos['categoria'];
		$this->RazonSocial = $aDatos['razon_social'];
		$this->IdCondicionfiscal = $aDatos['id_condicionfiscal'];
		$this->Cuit = $aDatos['cuit'];
		$this->Domicilio = $aDatos['domicilio'];
		$this->DomicilioComercial = $aDatos['domicilio_comercial'];
		$this->DomicilioCorrespondencia = $aDatos['domicilio_correspondencia'];
		$this->Telefono = $aDatos['telefono'];
		$this->PaginaWeb = $aDatos['pagina_web'];
		$this->Email = $aDatos['email'];
		$this->IdComercial = $aDatos['id_comercial'];
		$this->Bloqueado = $aDatos['bloqueado'];
		$this->Comentario = $aDatos['comentario'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdCliente = $this->Nombre = $this->Categoria = $this->RazonSocial = $this->IdCondicionfiscal = $this->Cuit = $this->Domicilio = $this->DomicilioComercial = $this->DomicilioCorrespondencia = $this->Telefono = $this->PaginaWeb = $this->Email = $this->IdComercial = $this->Bloqueado = $this->Comentario = null;
	}
}
?>