<?
class Cobranzas extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdCobranza, $Fecha, $IdCliente, $IdMoneda, $Importe, $Notas;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Cobranzas.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Fecha !== null) {
			$aWhere[] = "cobranzas.fecha = '{$this->Fecha}'";
		}
		if (@$this->Desde !== null) {
			$aWhere[] = "cobranzas.fecha >= '{$this->Desde}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "cobranzas.fecha <= '{$this->Hasta}'";
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "cobranzas.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdMoneda !== null) {
			$aWhere[] = "cobranzas.id_moneda = '{$this->IdMoneda}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS cobranzas.*, clientes.razon_social, clientes.nombre cliente, monedas.nombre moneda, monedas.formato
			FROM cobranzas 
			INNER JOIN elementos ON elementos.id_entidad = cobranzas.id_cobranza AND elementos.id_tipoelemento = ".ELEMENTOS_COBRANZA." AND elementos.eliminado = 0 
			LEFT JOIN clientes ON clientes.id_cliente = cobranzas.id_cliente
			LEFT JOIN monedas ON monedas.id_moneda = cobranzas.id_moneda
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Cobranzas con ID $this->IdCobranza.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT cobranzas.*, clientes.razon_social, clientes.nombre cliente, monedas.nombre moneda
			FROM cobranzas 
			LEFT JOIN clientes ON clientes.id_cliente = cobranzas.id_cliente
			LEFT JOIN monedas ON monedas.id_moneda = cobranzas.id_moneda
			WHERE cobranzas.id_cobranza = '{$this->IdCobranza}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	public function Saldo() {
		$cSql = "SELECT SUM(cobranzas.importe) saldo 
			FROM cobranzas
			INNER JOIN elementos ON elementos.id_entidad = cobranzas.id_cobranza AND elementos.id_tipoelemento = ".ELEMENTOS_COBRANZA." AND elementos.eliminado = 0
			WHERE cobranzas.id_cliente = '{$this->IdCliente}'";
		$this->DB->Query($cSql);
		
		return $this->DB->Field('saldo');
	}
	
	public function SaldoGeneral() {
		$cSql = "
		SELECT SUM(cobranzas.importe * (
				SELECT importe FROM cotizaciones WHERE cotizaciones.id_moneda = cobranzas.id_moneda AND cotizaciones.fecha <= cobranzas.fecha ORDER BY cotizaciones.fecha DESC LIMIT 1
			)) saldo 
			FROM cobranzas
			INNER JOIN elementos ON elementos.id_entidad = cobranzas.id_cobranza AND elementos.id_tipoelemento = ".ELEMENTOS_COBRANZA." AND elementos.eliminado = 0
			WHERE cobranzas.fecha >= '{$this->Desde}' AND cobranzas.fecha <= '{$this->Hasta}'
			";
		$this->DB->Query($cSql);
		return $this->DB->Field('saldo');
	}
	
	/**
	 * @desc
	 * Guarda un Cobranzas.
	 * Devuelve el ID del Cobranzas guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdCobranza) {
			$cSql = "UPDATE cobranzas SET ".
				"fecha = '{$this->Fecha}', ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_moneda = '{$this->IdMoneda}', ".
				"importe = '{$this->Importe}', ".
				"notas = ".($this->Notas ? "'{$this->Notas}'" : 'NULL')." ".
				"WHERE id_cobranza = '{$this->IdCobranza}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO cobranzas SET ".
				"fecha = '{$this->Fecha}', ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_moneda = '{$this->IdMoneda}', ".
				"importe = '{$this->Importe}', ".
				"notas = ".($this->Notas ? "'{$this->Notas}'" : 'NULL');
			
			$this->IdCobranza = $this->DB->QueryInsert($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_COBRANZA, $this->IdCobranza);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdCobranza;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Cobranzas.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		Utils::EliminarElemento(ELEMENTOS_COBRANZA, $this->IdCobranza);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdCobranza = $aDatos['id_cobranza'];
		$this->Fecha = $aDatos['fecha'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->Importe = $aDatos['importe'];
		$this->IdMoneda = $aDatos['id_moneda'];
		$this->Notas = $aDatos['notas'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdCobranza = $this->Fecha = $this->IdCliente = $this->IdMoneda = $this->Importe = $this->Notas = null;
	}
}
?>