<?
class Condiciones_fiscales extends Model {
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdCondicionfiscal, $Nombre, $Letra, $Iva, $Grava, $Identificador;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Nombre !== null) {
			$aWhere[] = "condiciones_fiscales.nombre LIKE '{$this->Nombre}%'";
		}
		if ($this->Letra !== null) {
			$aWhere[] = "condiciones_fiscales.letra = '{$this->Letra}'";
		}
		if ($this->Iva !== null) {
			$aWhere[] = "condiciones_fiscales.iva = '{$this->Iva}'";
		}
		if ($this->Grava !== null) {
			$aWhere[] = "condiciones_fiscales.grava = '{$this->Grava}'";
		}
		if ($this->Identificador !== null) {
			$aWhere[] = "condiciones_fiscales.identificador = '{$this->Identificador}'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "condiciones_fiscales.nombre REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "condiciones_fiscales.nombre LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS condiciones_fiscales.* FROM condiciones_fiscales {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT condiciones_fiscales.* FROM condiciones_fiscales WHERE condiciones_fiscales.id_condicionfiscal = '{$this->IdCondicionfiscal}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdCondicionfiscal) {
			$cSql = "UPDATE condiciones_fiscales SET ".
				"nombre = '{$this->Nombre}', ".
				"letra = '{$this->Letra}', ".
				"iva = '{$this->Iva}', ".
				"grava = '{$this->Grava}', ".
				"identificador = '{$this->Identificador}' ".
				"WHERE id_condicionfiscal = '{$this->IdCondicionfiscal}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO condiciones_fiscales SET ".
				"nombre = '{$this->Nombre}', ".
				"letra = '{$this->Letra}', ".
				"iva = '{$this->Iva}', ".
				"grava = '{$this->Grava}', ".
				"identificador = '{$this->Identificador}'";
			
			$this->IdCondicionfiscal = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdCondicionfiscal;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM condiciones_fiscales WHERE id_condicionfiscal = '{$this->IdCondicionfiscal}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdCondicionfiscal = $aDatos['id_condicionfiscal'];
		$this->Nombre = $aDatos['nombre'];
		$this->Letra = $aDatos['letra'];
		$this->Iva = $aDatos['iva'];
		$this->Grava = $aDatos['grava'];
		$this->Identificador = $aDatos['identificador'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdCondicionfiscal = $this->Nombre = $this->Letra = $this->Iva = $this->Grava = $this->Identificador = null;
	}
}
?>