<?
class Cotizaciones extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdCotizacion, $Fecha, $IdMoneda, $Importe;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Cotizaciones.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Fecha !== null) {
			$aWhere[] = "cotizaciones.fecha = '{$this->Fecha}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "cotizaciones.fecha <= '{$this->Hasta}'";
		}
		if ($this->IdMoneda !== null) {
			$aWhere[] = "cotizaciones.id_moneda = '{$this->IdMoneda}'";
		}
		if ($this->Importe !== null) {
			$aWhere[] = "cotizaciones.importe = '{$this->Importe}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS cotizaciones.*, monedas.nombre moneda, monedas.formato
			FROM cotizaciones 
			INNER JOIN elementos ON elementos.id_entidad = cotizaciones.id_cotizacion AND elementos.id_tipoelemento = ".ELEMENTOS_COTIZACION." AND elementos.eliminado = 0 
			LEFT JOIN monedas ON monedas.id_moneda = cotizaciones.id_moneda
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Cotizaciones con ID $this->IdCotizacion.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT cotizaciones.*, monedas.nombre moneda
			FROM cotizaciones 
			LEFT JOIN monedas ON monedas.id_moneda = cotizaciones.id_moneda
			WHERE cotizaciones.id_cotizacion = '{$this->IdCotizacion}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Cotizaciones.
	 * Devuelve el ID del Cotizaciones guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdCotizacion) {
			$cSql = "UPDATE cotizaciones SET ".
				"fecha = '{$this->Fecha}', ".
				"id_moneda = '{$this->IdMoneda}', ".
				"importe = '{$this->Importe}' ".
				"WHERE id_cotizacion = '{$this->IdCotizacion}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO cotizaciones SET ".
				"fecha = '{$this->Fecha}', ".
				"id_moneda = '{$this->IdMoneda}', ".
				"importe = '{$this->Importe}'";
			
			$this->IdCotizacion = $this->DB->QueryInsert($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_COTIZACION, $this->IdCotizacion);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdCotizacion;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Cotizaciones.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		Utils::EliminarElemento(ELEMENTOS_COTIZACION, $this->IdCotizacion);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdCotizacion = $aDatos['id_cotizacion'];
		$this->Fecha = $aDatos['fecha'];
		$this->IdMoneda = $aDatos['id_moneda'];
		$this->Importe = $aDatos['importe'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdCotizacion = $this->Fecha = $this->IdMoneda = $this->Importe = null;
	}
}
?>