<?
class Cronjobs extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdCronjob, $Tarea, $Descripcion, $UltimaEjecucion, $PeriodoEjecucion;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Cronjobs.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Tarea !== null) {
			$aWhere[] = "cronjobs.tarea = '{$this->Tarea}'";
		}
		if ($this->Descripcion !== null) {
			$aWhere[] = "cronjobs.descripcion = '{$this->Descripcion}'";
		}
		if ($this->UltimaEjecucion !== null) {
			$aWhere[] = "cronjobs.ultima_ejecucion = '{$this->UltimaEjecucion}'";
		}
		if ($this->PeriodoEjecucion !== null) {
			$aWhere[] = "cronjobs.periodo_ejecucion = '{$this->PeriodoEjecucion}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS cronjobs.* FROM cronjobs {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Cronjobs con ID $this->IdCronjob.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT cronjobs.* FROM cronjobs WHERE cronjobs.id_cronjob = '{$this->IdCronjob}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Cronjobs.
	 * Devuelve el ID del Cronjobs guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdCronjob) {
			$cSql = "UPDATE cronjobs SET ".
				"tarea = ".($this->Tarea ? "'{$this->Tarea}'" : 'NULL').", ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"ultima_ejecucion = '{$this->UltimaEjecucion}', ".
				"periodo_ejecucion = '{$this->PeriodoEjecucion}' ".
				"WHERE id_cronjob = '{$this->IdCronjob}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO cronjobs SET ".
				"tarea = ".($this->Tarea ? "'{$this->Tarea}'" : 'NULL').", ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"ultima_ejecucion = '{$this->UltimaEjecucion}', ".
				"periodo_ejecucion = '{$this->PeriodoEjecucion}'";
			
			$this->IdCronjob = $this->DB->QueryInsert($cSql);
			
			//Utils::GuardarElemento(ELEMENTOS_CRONJOB, $this->IdCronjob);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdCronjob;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Cronjobs.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		//Utils::EliminarElemento(ELEMENTOS_CRONJOB, $this->IdCronjob);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdCronjob = $aDatos['id_cronjob'];
		$this->Tarea = $aDatos['tarea'];
		$this->Descripcion = $aDatos['descripcion'];
		$this->UltimaEjecucion = $aDatos['ultima_ejecucion'];
		$this->PeriodoEjecucion = $aDatos['periodo_ejecucion'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdCronjob = $this->Tarea = $this->Descripcion = $this->UltimaEjecucion = $this->PeriodoEjecucion = null;
	}
}
?>