<?
class CuentasCorrientes extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdCliente;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Cuentas corrientes.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdCliente !== null) {
			$aWhere[] = "id_cliente = '{$this->IdCliente}'";
		}
		
		if (@$this->Saldos === true) {
			$cSelect = 'origen.id_cliente, clientes.nombre cliente, SUM(origen.haber - origen.debe) saldo';
			$cGroupBy = 'GROUP BY clientes.id_cliente HAVING saldo <> 0';
		}
		else {
			$cSelect = 'origen.*, clientes.nombre cliente';
			$cGroupBy = '';
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "
			SELECT {$cSelect} FROM (
				(
					SELECT facturas.id_factura id, tipos_factura.nombre tipo, facturas.comprobante comprobante, facturas.fecha, facturas.id_cliente, facturas.total debe, 0 haber
					FROM facturas
					LEFT JOIN tipos_factura ON tipos_factura.id_tipofactura = facturas.id_tipofactura
					{$cWhere}
				)
				UNION (
					SELECT cobranzas.id_cobranza id, '' tipo,  cobranzas.id_cobranza comprobante, cobranzas.fecha, cobranzas.id_cliente, 0 debe, cobranzas.importe haber
					FROM cobranzas
					INNER JOIN elementos ON elementos.id_entidad = cobranzas.id_cobranza AND elementos.id_tipoelemento = ".ELEMENTOS_COBRANZA." AND elementos.eliminado = 0
					{$cWhere}
				)
			) AS origen
			LEFT JOIN clientes ON clientes.id_cliente = origen.id_cliente
			{$cGroupBy} {$cOrderBy}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		/*
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		*/
		return $aRet;
	}
}
?>