<?
class Destinatarios_seguimiento extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdDestinatario, $IdSeguimiento;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdSeguimiento !== null) {
			$aWhere[] = "destinatarios_seguimiento.id_seguimiento = '{$this->IdSeguimiento}'";
		}
		if ($this->IdDestinatario !== null) {
			$aWhere[] = "destinatarios_seguimiento.id_destinatario = '{$this->IdDestinatario}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS destinatarios_seguimiento.* FROM destinatarios_seguimiento {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	public function BuscarNombres() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		$cOrderBy = "ORDER BY operadores.nombre";
		
		// Paginacion.
		$cLimit = '';
		
		if ($this->IdSeguimiento !== null) {
			$aWhere[] = "destinatarios_seguimiento.id_seguimiento = '{$this->IdSeguimiento}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT operadores.nombre FROM destinatarios_seguimiento LEFT JOIN operadores ON operadores.id_operador = destinatarios_seguimiento.id_destinatario {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		return join(', ', $this->DB->GetRecordset('nombre'));
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT destinatarios_seguimiento.* FROM destinatarios_seguimiento WHERE destinatarios_seguimiento.id_destinatario = '{$this->IdDestinatario}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$cSql = "INSERT IGNORE INTO destinatarios_seguimiento SET ".
			"id_seguimiento = '{$this->IdSeguimiento}', ".
			"id_destinatario = '{$this->IdDestinatario}'";
		
		return $this->DB->Query($cSql);
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM destinatarios_seguimiento WHERE id_seguimiento = '{$this->IdSeguimiento}' && id_destinatario = '{$this->IdDestinatario}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdDestinatario = $aDatos['id_destinatario'];
		$this->IdSeguimiento = $aDatos['id_seguimiento'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdDestinatario = $this->IdSeguimiento = null;
	}
}
?>