<?
class Detalles_factura extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdDetallefactura, $IdFactura, $Descripcion, $Importe;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Detalles_factura.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdFactura !== null) {
			$aWhere[] = "detalles_factura.id_factura = '{$this->IdFactura}'";
		}
		if ($this->Descripcion !== null) {
			$aWhere[] = "detalles_factura.descripcion = '{$this->Descripcion}'";
		}
		if ($this->Importe !== null) {
			$aWhere[] = "detalles_factura.importe = '{$this->Importe}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS detalles_factura.* FROM detalles_factura {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Detalles_factura con ID $this->IdDetallefactura.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT detalles_factura.* FROM detalles_factura WHERE detalles_factura.id_detallefactura = '{$this->IdDetallefactura}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Detalles_factura.
	 * Devuelve el ID del Detalles_factura guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdDetallefactura) {
			$cSql = "UPDATE detalles_factura SET ".
				"id_factura = '{$this->IdFactura}', ".
				"descripcion = '{$this->Descripcion}', ".
				"importe = '{$this->Importe}' ".
				"WHERE id_detallefactura = '{$this->IdDetallefactura}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO detalles_factura SET ".
				"id_factura = '{$this->IdFactura}', ".
				"descripcion = '{$this->Descripcion}', ".
				"importe = '{$this->Importe}'";
			
			$this->IdDetallefactura = $this->DB->QueryInsert($cSql);
			
			//Utils::GuardarElemento(ELEMENTOS_DETALLES_FACTURA, $this->IdDetallefactura);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdDetallefactura;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Detalles_factura.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		//Utils::EliminarElemento(ELEMENTOS_DETALLES_FACTURA, $this->IdDetallefactura);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdDetallefactura = $aDatos['id_detallefactura'];
		$this->IdFactura = $aDatos['id_factura'];
		$this->Descripcion = $aDatos['descripcion'];
		$this->Importe = $aDatos['importe'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdDetallefactura = $this->IdFactura = $this->Descripcion = $this->Importe = null;
	}
}
?>