<?
class Egresos extends Model {
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdEgreso, $Fecha, $IdTipoegreso, $IdProyecto, $IdOperador, $Descripcion, $Computar, $IdProveedor, $IdTipofactura, $Comprobante, $Importe, $Nogravado, $Iva, $PercepcionIva, $PercepcionIIBB, $Total, $Notas;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Fecha !== null) {
			$aWhere[] = "egresos.fecha = '{$this->Fecha}'";
		}
		if (@$this->Desde !== null) {
			$aWhere[] = "egresos.fecha >= '{$this->Desde}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "egresos.fecha <= '{$this->Hasta}'";
		}
		if ($this->IdTipoegreso !== null) {
			$aWhere[] = "egresos.id_tipoegreso = '{$this->IdTipoegreso}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "egresos.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdOperador !== null) {
			$aWhere[] = "egresos.id_operador = '{$this->IdOperador}'";
		}
		if ($this->Computar !== null) {
			$aWhere[] = "egresos.computar = '{$this->Computar}'";
		}
		if ($this->IdProveedor !== null) {
			$aWhere[] = "egresos.id_proveedor = '{$this->IdProveedor}'";
		}
		if ($this->Descripcion !== null) {
			$aWhere[] = "egresos.descripcion LIKE '%{$this->Descripcion}%'";
		}
		if ($this->IdTipofactura !== null) {
			if ($this->IdTipofactura === true) {
				$aWhere[] = "egresos.id_tipofactura IS NOT NULL";
			}
			elseif ($this->IdTipofactura === false) {
				$aWhere[] = "egresos.id_tipofactura IS NULL";
			}
			else {
				$aWhere[] = "egresos.id_tipofactura = '{$this->IdTipofactura}'";
			}
		}
		if ($this->Comprobante !== null) {
			$aWhere[] = "egresos.comprobante = '{$this->Comprobante}'";
		}
		if (@$this->Factura !== null) {
			$aWhere[] = "egresos.id_tipofactura IS NOT NULL";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "egresos.descripcion REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "egresos.descripcion LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS egresos.*, tipos_egresos.nombre tipo_egreso, tipos_factura.nombre tipo_factura, proveedores.denominacion proveedor, proveedores.razon_social, proveedores.cuit
			FROM egresos 
			LEFT JOIN tipos_egresos ON tipos_egresos.id_tipoegreso = egresos.id_tipoegreso
			LEFT JOIN proveedores ON proveedores.id_proveedor = egresos.id_proveedor
			LEFT JOIN tipos_factura ON tipos_factura.id_tipofactura = egresos.id_tipofactura
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT egresos.*, tipos_egresos.nombre tipo_egreso, proyectos.nombre proyecto, operadores.nombre operador, tipos_factura.nombre tipo_factura, proveedores.denominacion proveedor, proveedores.razon_social, proveedores.cuit
			FROM egresos 
			LEFT JOIN tipos_egresos ON tipos_egresos.id_tipoegreso = egresos.id_tipoegreso
			LEFT JOIN proyectos ON proyectos.id_proyecto = egresos.id_proyecto
			LEFT JOIN operadores ON operadores.id_operador = egresos.id_operador
			LEFT JOIN proveedores ON proveedores.id_proveedor = egresos.id_proveedor
			LEFT JOIN tipos_factura ON tipos_factura.id_tipofactura = egresos.id_tipofactura
			WHERE egresos.id_egreso = '{$this->IdEgreso}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	public function Saldo() {
		$cSql = "SELECT SUM(egresos.total) saldo 
			FROM egresos
			WHERE egresos.fecha >= '{$this->Desde}' AND egresos.fecha <= '{$this->Hasta}'";
		$this->DB->Query($cSql);
		
		return $this->DB->Field('saldo');
	}
	
	/**
	 * @desc
	 * Guarda un Egresos.
	 * Devuelve el ID del Egresos guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdEgreso) {
			$cSql = "UPDATE egresos SET ".
				"fecha = '{$this->Fecha}', ".
				"id_tipoegreso = '{$this->IdTipoegreso}', ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"id_operador = ".($this->IdOperador ? "'{$this->IdOperador}'" : 'NULL').", ".
				"descripcion = '{$this->Descripcion}', ".
				"computar = '{$this->Computar}', ".
				"id_proveedor = ".($this->IdProveedor ? "'{$this->IdProveedor}'" : 'NULL').", ".
				"id_tipofactura = ".($this->IdTipofactura ? "'{$this->IdTipofactura}'" : 'NULL').", ".
				"comprobante = ".($this->Comprobante ? "'{$this->Comprobante}'" : 'NULL').", ".
				"importe = '{$this->Importe}', ".
				"nogravado = '{$this->Nogravado}', ".
				"iva = '{$this->Iva}', ".
				"percepcion_iva = '{$this->PercepcionIva}', ".
				"percepcion_iibb = '{$this->PercepcionIIBB}', ".
				"total = '{$this->Total}', ".
				"notas = ".($this->Notas ? "'{$this->Notas}'" : 'NULL')." ".
				"WHERE id_egreso = '{$this->IdEgreso}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO egresos SET ".
				"fecha = '{$this->Fecha}', ".
				"id_tipoegreso = '{$this->IdTipoegreso}', ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"id_operador = ".($this->IdOperador ? "'{$this->IdOperador}'" : 'NULL').", ".
				"descripcion = '{$this->Descripcion}', ".
				"computar = '{$this->Computar}', ".
				"id_proveedor = ".($this->IdProveedor ? "'{$this->IdProveedor}'" : 'NULL').", ".
				"id_tipofactura = ".($this->IdTipofactura ? "'{$this->IdTipofactura}'" : 'NULL').", ".
				"comprobante = ".($this->Comprobante ? "'{$this->Comprobante}'" : 'NULL').", ".
				"importe = '{$this->Importe}', ".
				"nogravado = '{$this->Nogravado}', ".
				"iva = '{$this->Iva}', ".
				"percepcion_iva = '{$this->PercepcionIva}', ".
				"percepcion_iibb = '{$this->PercepcionIIBB}', ".
				"total = '{$this->Total}', ".
				"notas = ".($this->Notas ? "'{$this->Notas}'" : 'NULL');
			
			$this->IdEgreso = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdEgreso;
		}
	}
	
	/**
	 * @desc
	 * Devuelve una sumatoria totalizada de los egresos.
	 *
	 */
	public function GranTotal() {
		$aWhere = array();
		
		if ($this->Computar !== null) {
			$aWhere[] = "computar = '{$this->Computar}'";
		}
		if ($this->IdTipofactura === true) {
			$aWhere[] = "id_tipofactura IS NOT NULL";
		}
		if ($this->IdTipofactura === false) {
			$aWhere[] = "id_tipofactura IS NULL";
		}
		
		$cWhere = $aWhere ? 'AND '.join(' AND ', $aWhere) : '';
		
		$cSql = "
			SELECT SUM(importe) importe, SUM(nogravado) nogravado, SUM(iva) iva, SUM(percepcion_iva) percepcion_iva, SUM(percepcion_iibb) percepcion_iibb, SUM(total) total
			FROM egresos
			WHERE (fecha BETWEEN '{$this->Desde}' AND '{$this->Hasta}') {$cWhere}
		";
		$this->DB->Query($cSql);
		return $this->DB->Field();
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM egresos WHERE id_egreso = '{$this->IdEgreso}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdEgreso = $aDatos['id_egreso'];
		$this->Fecha = $aDatos['fecha'];
		$this->IdTipoegreso = $aDatos['id_tipoegreso'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->IdOperador = $aDatos['id_operador'];
		$this->Descripcion = $aDatos['descripcion'];
		$this->Computar = $aDatos['computar'];
		$this->IdProveedor = $aDatos['id_proveedor'];
		$this->IdTipofactura = $aDatos['id_tipofactura'];
		$this->Comprobante = $aDatos['comprobante'];
		$this->Importe = $aDatos['importe'];
		$this->Nogravado = $aDatos['nogravado'];
		$this->Iva = $aDatos['iva'];
		$this->PercepcionIva = $aDatos['percepcion_iva'];
		$this->PercepcionIIBB = $aDatos['percepcion_iibb'];
		$this->Total = $aDatos['total'];
		$this->Notas = $aDatos['notas'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdEgreso = $this->Fecha = $this->IdTipoegreso = $this->IdProyecto = $this->IdOperador = $this->Descripcion = $this->Computar = $this->IdProveedor = $this->IdTipofactura = $this->Comprobante = $this->Importe = $this->Nogravado = $this->Iva = $this->PercepcionIva = $this->PercepcionIIBB = $this->Total = $this->Notas = null;
	}
}
?>