<?
class Facturas extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdFactura, $Fecha, $IdCliente, $IdProyecto, $IdCondicionfiscal, $IdTipofactura, $Comprobante, $Importe, $Iva, $Impuesto, $Total, $IdMoneda;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Facturas.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Fecha !== null) {
			$aWhere[] = "facturas.fecha = '{$this->Fecha}'";
		}
		if (@$this->Desde !== null) {
			$aWhere[] = "facturas.fecha >= '{$this->Desde}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "facturas.fecha <= '{$this->Hasta}'";
		}
		if (@$this->EnBlanco !== null) {
			$aWhere[] = "facturas.id_tipofactura IS NOT NULL";
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "facturas.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "facturas.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdTipofactura !== null) {
			$aWhere[] = "facturas.id_tipofactura = '{$this->IdTipofactura}'";
		}
		if ($this->IdCondicionfiscal !== null) {
			$aWhere[] = "facturas.id_condicionfiscal = '{$this->IdCondicionfiscal}'";
		}
		if ($this->Comprobante !== null) {
			$aWhere[] = "facturas.comprobante = '{$this->Comprobante}'";
		}
		if ($this->IdMoneda !== null) {
			$aWhere[] = "facturas.id_moneda = '{$this->IdMoneda}'";
		}
		if ($this->Importe !== null) {
			$aWhere[] = "facturas.importe = '{$this->Importe}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS facturas.*, clientes.razon_social, clientes.nombre cliente, clientes.cuit, monedas.nombre moneda, monedas.formato, tipos_factura.nombre tipo_factura, condiciones_fiscales.grava
			FROM facturas 
			LEFT JOIN clientes ON clientes.id_cliente = facturas.id_cliente
			LEFT JOIN condiciones_fiscales ON condiciones_fiscales.id_condicionfiscal = clientes.id_condicionfiscal
			LEFT JOIN monedas ON monedas.id_moneda = facturas.id_moneda
			LEFT JOIN tipos_factura ON tipos_factura.id_tipofactura = facturas.id_tipofactura
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Facturas con ID $this->IdFactura.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT facturas.*, clientes.razon_social, clientes.nombre cliente, monedas.nombre moneda, monedas.formato, tipos_factura.nombre tipo_factura, condiciones_fiscales.nombre condicion_fiscal, proyectos.nombre proyecto
			FROM facturas
			LEFT JOIN clientes ON clientes.id_cliente = facturas.id_cliente
			LEFT JOIN proyectos ON proyectos.id_proyecto = facturas.id_proyecto
			LEFT JOIN monedas ON monedas.id_moneda = facturas.id_moneda
			LEFT JOIN tipos_factura ON tipos_factura.id_tipofactura = facturas.id_tipofactura
			LEFT JOIN condiciones_fiscales ON condiciones_fiscales.id_condicionfiscal = facturas.id_condicionfiscal
			WHERE facturas.id_factura = '{$this->IdFactura}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	public function Saldo() {
		if ($this->IdCliente) {
			$cSql = "SELECT SUM(facturas.total) saldo 
				FROM facturas
				WHERE facturas.id_cliente = '{$this->IdCliente}'";
		}
		elseif (@$this->Desde) {
			$cSql = "SELECT SUM(facturas.total * (
					SELECT importe FROM cotizaciones WHERE cotizaciones.id_moneda = facturas.id_moneda AND cotizaciones.fecha <= facturas.fecha ORDER BY cotizaciones.fecha DESC LIMIT 1
				)) saldo 
				FROM facturas
				WHERE facturas.fecha >= '{$this->Desde}' AND facturas.fecha <= '{$this->Hasta}'";
		}
		
		$this->DB->Query($cSql);
		
		return $this->DB->Field('saldo');
	}
	
	/**
	 * @desc
	 * Guarda un Facturas.
	 * Devuelve el ID del Facturas guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdFactura) {
			$cSql = "UPDATE facturas SET ".
				"fecha = '{$this->Fecha}', ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_proyecto = '{$this->IdProyecto}', ".
				"id_condicionfiscal = ".($this->IdCondicionfiscal ? "'{$this->IdCondicionfiscal}'" : 'NULL').", ".
				"id_tipofactura = ".($this->IdTipofactura ? "'{$this->IdTipofactura}'" : 'NULL').", ".
				"comprobante = ".($this->Comprobante ? "'{$this->Comprobante}'" : 'NULL').", ".
				"id_moneda = '{$this->IdMoneda}', ".
				"importe = '{$this->Importe}', ".
				"iva = '{$this->Iva}', ".
				"impuesto = '{$this->Impuesto}', ".
				"total = '{$this->Total}' ".
				"WHERE id_factura = '{$this->IdFactura}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO facturas SET ".
				"fecha = '{$this->Fecha}', ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_proyecto = '{$this->IdProyecto}', ".
				"id_condicionfiscal = ".($this->IdCondicionfiscal ? "'{$this->IdCondicionfiscal}'" : 'NULL').", ".
				"id_tipofactura = ".($this->IdTipofactura ? "'{$this->IdTipofactura}'" : 'NULL').", ".
				"comprobante = ".($this->Comprobante ? "'{$this->Comprobante}'" : 'NULL').", ".
				"id_moneda = '{$this->IdMoneda}', ".
				"importe = '{$this->Importe}', ".
				"iva = '{$this->Iva}', ".
				"impuesto = '{$this->Impuesto}', ".
				"total = '{$this->Total}' ";
			
			$this->IdFactura = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdFactura;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Facturas.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		$cSql = "DELETE FROM facturas WHERE id_factura = '{$this->IdFactura}' LIMIT 1";
		$this->DB->Query($cSql);
		
		$cSql = "UPDATE ordenes SET id_factura = NULL WHERE id_factura = '{$this->IdFactura}'";
		$this->DB->Query($cSql);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Devuelve una sumatoria totalizada de la facturación.
	 *
	 */
	public function GranTotal() {
		$cSql = "
			SELECT SUM(importe*cotiza) importe, SUM(iva*cotiza) iva, SUM(total*cotiza) total
			FROM (
				SELECT facturas.*, (SELECT cotizaciones.importe FROM cotizaciones WHERE cotizaciones.fecha <= facturas.fecha AND cotizaciones.id_moneda = facturas.id_moneda ORDER BY fecha DESC LIMIT 1) cotiza
				FROM facturas
				WHERE facturas.id_tipofactura IS NOT NULL AND (facturas.fecha BETWEEN '{$this->Desde}' AND '{$this->Hasta}')
			) subq
		";
		
		/*
		$cSql = "
			SELECT SUM(importe) importe, SUM(iva) iva, SUM(total) total
			FROM facturas
			WHERE id_tipofactura IS NOT NULL AND (fecha BETWEEN '{$this->Desde}' AND '{$this->Hasta}')
		";*/
		
		$this->DB->Query($cSql);
		$aRet['declarada'] = $this->DB->Field();
		
		$cSql = "
			SELECT SUM(importe*cotiza) importe, SUM(iva*cotiza) iva, SUM(total*cotiza) total
			FROM (
				SELECT facturas.*, (SELECT cotizaciones.importe FROM cotizaciones WHERE cotizaciones.fecha <= facturas.fecha AND cotizaciones.id_moneda = facturas.id_moneda ORDER BY fecha DESC LIMIT 1) cotiza
				FROM facturas
				WHERE facturas.id_tipofactura IS NULL AND (facturas.fecha BETWEEN '{$this->Desde}' AND '{$this->Hasta}')
			) subq
		";
		/*
		$cSql = "
			SELECT SUM(importe) importe, SUM(iva) iva, SUM(total) total
			FROM facturas
			WHERE id_tipofactura IS NULL AND (fecha BETWEEN '{$this->Desde}' AND '{$this->Hasta}')
		";
		*/
		$this->DB->Query($cSql);
		$aRet['no_declarada'] = $this->DB->Field();
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdFactura = $aDatos['id_factura'];
		$this->Fecha = $aDatos['fecha'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->IdCondicionfiscal = $aDatos['id_condicionfiscal'];
		$this->IdTipofactura = $aDatos['id_tipofactura'];
		$this->Comprobante = $aDatos['comprobante'];
		$this->IdMoneda = $aDatos['id_moneda'];
		$this->Importe = $aDatos['importe'];
		$this->Iva = $aDatos['iva'];
		$this->Impuesto = $aDatos['impuesto'];
		$this->Total = $aDatos['total'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdFactura = $this->Fecha = $this->IdCliente = $this->IdProyecto = $this->IdTipofactura = $this->IdCondicionfiscal = $this->Comprobante = $this->IdMoneda = $this->Importe = $this->Iva = $this->Impuesto = $this->Total = null;
	}
}
?>