<?
class Fases_proyecto extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdFaseproyecto, $IdProyecto, $IdFase, $InicioEstimado, $FinEstimado, $InicioReal, $FinReal;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdProyecto !== null) {
			$aWhere[] = "fases_proyecto.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdFase !== null) {
			$aWhere[] = "fases_proyecto.id_fase = '{$this->IdFase}'";
		}
		if ($this->InicioEstimado !== null) {
			$aWhere[] = "fases_proyecto.inicio_estimado = '{$this->InicioEstimado}'";
		}
		if ($this->FinEstimado !== null) {
			$aWhere[] = "fases_proyecto.fin_estimado = '{$this->FinEstimado}'";
		}
		if ($this->InicioReal !== null) {
			$aWhere[] = "fases_proyecto.inicio_real = '{$this->InicioReal}'";
		}
		if ($this->FinReal !== null) {
			$aWhere[] = "fases_proyecto.fin_real = '{$this->FinReal}'";
		}
		if (isset($this->FiltroActivos)) {
			$aWhere[] = "fases_proyecto.inicio_real <= '{$this->FiltroActivos}'";
			$aWhere[] = "fases_proyecto.fin_real IS NULL";
		}
		if (isset($this->FiltroPendientes)) {
			$aWhere[] = "fases_proyecto.inicio_estimado <= DATE_ADD('{$this->FiltroPendientes}', INTERVAL 30 DAY)";
			$aWhere[] = "fases_proyecto.inicio_real IS NULL";
		}
		if (isset($this->FiltroEnCola)) {
			$aWhere[] = "fases_proyecto.fin_real IS NULL";
		}
		if (isset($this->FiltroTerminados)) {
			$aWhere[] = "fases_proyecto.fin_real IS NOT NULL";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS fases_proyecto.*, proyectos.nombre proyecto, proyectos.fin_estimado proyecto_fin_estimado, clientes.nombre cliente, fases.nombre fase, tipos_proyecto.nombre tipo_proyecto
			FROM fases_proyecto 
			LEFT JOIN proyectos ON proyectos.id_proyecto = fases_proyecto.id_proyecto
			LEFT JOIN clientes ON clientes.id_cliente = proyectos.id_cliente
			LEFT JOIN fases ON fases.id_fase = fases_proyecto.id_fase
			LEFT JOIN tipos_proyecto ON tipos_proyecto.id_tipoproyecto = proyectos.id_tipoproyecto
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		if ($this->IdFaseproyecto) {
			$cSql = "SELECT fases_proyecto.* FROM fases_proyecto WHERE fases_proyecto.id_faseproyecto = '{$this->IdFaseproyecto}' LIMIT 1";
		}
		else {
			$cSql = "SELECT fases_proyecto.* FROM fases_proyecto WHERE fases_proyecto.id_fase = '{$this->IdFase}' AND fases_proyecto.id_proyecto = '{$this->IdProyecto}' LIMIT 1";
		}
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdFaseproyecto) {
			$cSql = "UPDATE fases_proyecto SET ".
				"id_proyecto = '{$this->IdProyecto}', ".
				"id_fase = '{$this->IdFase}', ".
				"inicio_estimado = '{$this->InicioEstimado}', ".
				"fin_estimado = '{$this->FinEstimado}', ".
				"inicio_real = ".($this->InicioReal ? "'{$this->InicioReal}'" : 'NULL').", ".
				"fin_real = ".($this->FinReal ? "'{$this->FinReal}'" : 'NULL')." ".
				"WHERE id_faseproyecto = '{$this->IdFaseproyecto}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO fases_proyecto SET ".
				"id_proyecto = '{$this->IdProyecto}', ".
				"id_fase = '{$this->IdFase}', ".
				"inicio_estimado = '{$this->InicioEstimado}', ".
				"fin_estimado = '{$this->FinEstimado}', ".
				"inicio_real = ".($this->InicioReal ? "'{$this->InicioReal}'" : 'NULL').", ".
				"fin_real = ".($this->FinReal ? "'{$this->FinReal}'" : 'NULL')."";
			
			$this->IdFaseproyecto = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdFaseproyecto;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM fases_proyecto WHERE id_faseproyecto = '{$this->IdFaseproyecto}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdFaseproyecto = $aDatos['id_faseproyecto'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->IdFase = $aDatos['id_fase'];
		$this->InicioEstimado = $aDatos['inicio_estimado'];
		$this->FinEstimado = $aDatos['fin_estimado'];
		$this->InicioReal = $aDatos['inicio_real'];
		$this->FinReal = $aDatos['fin_real'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdFaseproyecto = $this->IdProyecto = $this->IdFase = $this->InicioEstimado = $this->FinEstimado = $this->InicioReal = $this->FinReal = null;
	}
}
?>