<?
class Historial_trabajo extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdHistorial, $Fecha, $IdOperador, $IdCliente, $IdProyecto, $Tiempo, $Nota;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cSelect = $cOrderBy = $cGroupBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Fecha !== null) {
			$aWhere[] = "historial_trabajo.fecha = '{$this->Fecha}'";
		}
		if (@$this->Desde !== null) {
			$aWhere[] = "historial_trabajo.fecha >= '{$this->Desde}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "historial_trabajo.fecha <= '{$this->Hasta}'";
		}
		if ($this->IdOperador !== null) {
			$aWhere[] = "historial_trabajo.id_operador = '{$this->IdOperador}'";
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "historial_trabajo.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "historial_trabajo.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->Tiempo !== null) {
			$aWhere[] = "historial_trabajo.tiempo = '{$this->Tiempo}'";
		}
		if ($this->Nota !== null) {
			$aWhere[] = "historial_trabajo.nota = '{$this->Nota}'";
		}
		if (@$this->PorOperador) {
			$cSelect = ", SUM(TIME_TO_SEC(historial_trabajo.tiempo)) tiempo_total";
			$cGroupBy = "GROUP BY historial_trabajo.id_operador, historial_trabajo.id_proyecto, historial_trabajo.id_cliente";
		}
		if (@$this->PorCliente) {
			$cSelect = ", SUM(TIME_TO_SEC(historial_trabajo.tiempo)) tiempo_total";
			$cGroupBy = "GROUP BY historial_trabajo.id_cliente";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "
			SELECT SQL_CALC_FOUND_ROWS historial_trabajo.*, TIME_TO_SEC(historial_trabajo.tiempo) segundos, clientes.nombre cliente, clientes.categoria categoria_cliente, proyectos.nombre proyecto, operadores.nombre operador {$cSelect}
			FROM historial_trabajo 
				LEFT JOIN clientes ON clientes.id_cliente = historial_trabajo.id_cliente
				LEFT JOIN proyectos ON proyectos.id_proyecto = historial_trabajo.id_proyecto
				LEFT JOIN operadores ON operadores.id_operador = historial_trabajo.id_operador
			{$cWhere} {$cGroupBy} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function SumaHoras() {
		$cSelect = $cOrderBy = $cGroupBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Fecha !== null) {
			$aWhere[] = "historial_trabajo.fecha = '{$this->Fecha}'";
		}
		if (@$this->Desde !== null) {
			$aWhere[] = "historial_trabajo.fecha >= '{$this->Desde}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "historial_trabajo.fecha <= '{$this->Hasta}'";
		}
		if ($this->IdOperador !== null) {
			$aWhere[] = "historial_trabajo.id_operador = '{$this->IdOperador}'";
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "historial_trabajo.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "historial_trabajo.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->Tiempo !== null) {
			$aWhere[] = "historial_trabajo.tiempo = '{$this->Tiempo}'";
		}
		if ($this->Nota !== null) {
			$aWhere[] = "historial_trabajo.nota = '{$this->Nota}'";
		}
		if (@$this->PorOperador) {
			$cSelect = ", SUM(TIME_TO_SEC(historial_trabajo.tiempo)) tiempo_total";
			$cGroupBy = "GROUP BY historial_trabajo.id_operador, historial_trabajo.id_proyecto, historial_trabajo.id_cliente";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "
			SELECT SUM(TIME_TO_SEC(historial_trabajo.tiempo)) segundos
			FROM historial_trabajo 
				LEFT JOIN clientes ON clientes.id_cliente = historial_trabajo.id_cliente
				LEFT JOIN proyectos ON proyectos.id_proyecto = historial_trabajo.id_proyecto
				LEFT JOIN operadores ON operadores.id_operador = historial_trabajo.id_operador
			{$cWhere} {$cGroupBy}";
		$this->DB->Query($cSql);
		return $this->DB->Field('segundos');
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "
			SELECT historial_trabajo.*, clientes.nombre cliente, proyectos.nombre proyecto, operadores.nombre operador
			FROM historial_trabajo 
				LEFT JOIN clientes ON clientes.id_cliente = historial_trabajo.id_cliente
				LEFT JOIN proyectos ON proyectos.id_proyecto = historial_trabajo.id_proyecto
				LEFT JOIN operadores ON operadores.id_operador = historial_trabajo.id_operador
			WHERE historial_trabajo.id_historial = '{$this->IdHistorial}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdHistorial) {
			$cSql = "UPDATE historial_trabajo SET ".
				"fecha = '{$this->Fecha}', ".
				"id_operador = '{$this->IdOperador}', ".
				"id_cliente = ".($this->IdCliente ? "'{$this->IdCliente}'" : 'NULL').", ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"tiempo = '{$this->Tiempo}', ".
				"nota = ".($this->Nota ? "'{$this->Nota}'" : 'NULL')." ".
				"WHERE id_historial = '{$this->IdHistorial}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO historial_trabajo SET ".
				"fecha = '{$this->Fecha}', ".
				"id_operador = '{$this->IdOperador}', ".
				"id_cliente = ".($this->IdCliente ? "'{$this->IdCliente}'" : 'NULL').", ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"tiempo = '{$this->Tiempo}', ".
				"nota = ".($this->Nota ? "'{$this->Nota}'" : 'NULL')."";
			
			$this->IdHistorial = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdHistorial;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM historial_trabajo WHERE id_historial = '{$this->IdHistorial}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdHistorial = $aDatos['id_historial'];
		$this->Fecha = $aDatos['fecha'];
		$this->IdOperador = $aDatos['id_operador'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->Tiempo = $aDatos['tiempo'];
		$this->Nota = $aDatos['nota'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdHistorial = $this->Fecha = $this->IdOperador = $this->IdCliente = $this->IdProyecto = $this->Tiempo = $this->Nota = null;
	}
}
?>