<?
class Operadores extends Model {
	
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdOperador, $Nombre, $Domicilio, $Localidad, $Telefono, $Email, $Contacto, $Usuario, $Clave, $Comentario, $ControlHistorial, $Proyectosencola, $Vencimientos, $Servicios, $Habilitado;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de operadores.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Nombre !== null) {
			$aWhere[] = "operadores.nombre LIKE '{$this->Nombre}%'";
		}
		if ($this->Domicilio !== null) {
			$aWhere[] = "operadores.domicilio = '{$this->Domicilio}'";
		}
		if ($this->Localidad !== null) {
			$aWhere[] = "operadores.localidad = '{$this->Localidad}'";
		}
		if ($this->Telefono !== null) {
			$aWhere[] = "operadores.telefono = '{$this->Telefono}'";
		}
		if ($this->Email !== null) {
			$aWhere[] = "operadores.email = '{$this->Email}'";
		}
		if ($this->Usuario !== null) {
			$aWhere[] = "operadores.usuario = '{$this->Usuario}'";
		}
		if ($this->Clave !== null) {
			$aWhere[] = "operadores.clave = '{$this->Clave}'";
		}
		if ($this->Comentario !== null) {
			$aWhere[] = "operadores.comentario = '{$this->Comentario}'";
		}
		if ($this->ControlHistorial !== null) {
			$aWhere[] = "operadores.control_historial = '{$this->ControlHistorial}'";
		}
		if ($this->Proyectosencola !== null) {
			$aWhere[] = "operadores.proy_en_cola = '{$this->Proyectosencola}'";
		}
		if ($this->Vencimientos !== null) {
			$aWhere[] = "operadores.vencimientos = '{$this->Vencimientos}'";
		}
		if ($this->Servicios !== null) {
			$aWhere[] = "operadores.servicios = '{$this->Servicios}'";
		}
		if ($this->Habilitado !== null) {
			$aWhere[] = "operadores.habilitado = '{$this->Habilitado}'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "operadores.nombre REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
		
			default:
				$aWhere[] = "operadores.nombre LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS operadores.* FROM operadores INNER JOIN elementos ON elementos.id_entidad = operadores.id_operador AND elementos.id_tipoelemento = ".ELEMENTOS_OPERADOR." AND elementos.eliminado = 0 {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Operadores con ID $this->IdOperador.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT operadores.* FROM operadores WHERE operadores.id_operador = '{$this->IdOperador}' LIMIT 1";
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		$cSql = "SELECT * FROM operadores_permisos WHERE id_operador = '{$this->IdOperador}' LIMIT 1";
		$this->DB->Query($cSql);
		$aRet['permisos'] = $this->DB->Field();
		
		$this->Set($aRet);
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el operador con usuario $cUsuario.
	 *
	 * @param string $cUsuario
	 * @return array
	 */
	public function ObtenerPorUsuario($cUsuario) {
		$cSql = "SELECT operadores.* FROM operadores INNER JOIN elementos ON elementos.id_tipoelemento = ".ELEMENTOS_OPERADOR." AND operadores.id_operador = elementos.id_entidad AND elementos.eliminado = 0 WHERE operadores.usuario = '{$cUsuario}' LIMIT 1";
		$this->DB->Query($cSql);
		
		if ($this->DB->Count()) {
			$aRet = $this->DB->Field();
			
			$cSql = "SELECT * FROM operadores_permisos WHERE id_operador = '{$aRet['id_operador']}' LIMIT 1";
			$this->DB->Query($cSql);
			$aRet['permisos'] = $this->DB->Field();
			$this->Set($aRet);
		}
		else {
			$aRet = array();
			$this->Limpiar();
		}
		
		
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un usuario.
	 * Devuelve el ID del operador guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdOperador) {
			$cSql = "UPDATE operadores SET ".
				"nombre = ".($this->Nombre ? "'{$this->Nombre}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"localidad = ".($this->Localidad ? "'{$this->Localidad}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"contacto = ".($this->Contacto ? "'{$this->Contacto}'" : 'NULL').", ".
				"usuario = '{$this->Usuario}', ".
				"clave = ".($this->Clave ? "'{$this->Clave}'" : 'NULL').", ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL').", ".
				"control_historial = '{$this->ControlHistorial}', ".
				"proy_en_cola = '{$this->Proyectosencola}', ".
				"vencimientos = '{$this->Vencimientos}', ".
				"servicios = '{$this->Servicios}', ".
				"habilitado = '{$this->Habilitado}' ".
				"WHERE id_operador = '{$this->IdOperador}' LIMIT 1";
			
			$this->DB->Query($cSql);
			
			$cSql = @"UPDATE operadores_permisos SET ".
				"id_operador = {$this->IdOperador}, ".
				@"administrador = '{$this->Permisos['administrador']}', ".
				@"facturacion = '{$this->Permisos['facturacion']}', ".
				@"ventas = '{$this->Permisos['ventas']}', ".
				@"desarrollo = '{$this->Permisos['desarrollo']}', ".
				@"freelance = '{$this->Permisos['freelance']}', ".
				@"gestion = '{$this->Permisos['gestion']}' ".
				"WHERE id_operador = '{$this->IdOperador}' LIMIT 1";
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO operadores SET ".
				"nombre = ".($this->Nombre ? "'{$this->Nombre}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"localidad = ".($this->Localidad ? "'{$this->Localidad}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"contacto = ".($this->Contacto ? "'{$this->Contacto}'" : 'NULL').", ".
				"usuario = '{$this->Usuario}', ".
				"clave = ".($this->Clave ? "'{$this->Clave}'" : 'NULL').", ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL').", ".
				"control_historial = '{$this->ControlHistorial}', ".
				"proy_en_cola = '{$this->Proyectosencola}', ".
				"vencimientos = '{$this->Vencimientos}', ".
				"servicios = '{$this->Servicios}', ".
				"habilitado = '{$this->Habilitado}' ";
			
			$this->IdOperador = $this->DB->QueryInsert($cSql);
			
			$cSql = @"INSERT INTO operadores_permisos SET ".
				"id_operador = {$this->IdOperador}, ".
				@"administrador = '{$this->Permisos['administrador']}', ".
				@"facturacion = '{$this->Permisos['facturacion']}', ".
				@"ventas = '{$this->Permisos['ventas']}', ".
				@"desarrollo = '{$this->Permisos['desarrollo']}', ".
				@"freelance = '{$this->Permisos['freelance']}', ".
				@"gestion = '{$this->Permisos['gestion']}' ";
			$this->DB->Query($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_OPERADOR, $this->IdOperador);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdOperador;
		}
	}
	
	/**
	 * Recibe un array con datos y lo aplica a todas las propiedades.
	 *
	 * @param array $aData
	 */
	public function Set($aData) {
		$this->IdOperador = @$aData['id_operador'];
		$this->Nombre = @$aData['nombre'];
		$this->Domicilio = @$aData['domicilio'];
		$this->Localidad = @$aData['localidad'];
		$this->Telefono = @$aData['telefono'];
		$this->Email = @$aData['email'];
		$this->Contacto = @$aData['contacto'];
		$this->Usuario = @$aData['usuario'];
		$this->Clave = @$aData['clave'];
		$this->Comentario = @$aData['comentario'];
		$this->ControlHistorial = @$aData['control_historial'];
		$this->Proyectosencola = @$aData['proy_en_cola'];
		$this->Vencimientos = @$aData['vencimientos'];
		$this->Servicios = @$aData['servicios'];
		$this->Habilitado = @$aData['habilitado'];
	}
	
	/**
	 * @desc
	 * Elimina un operador.
	 *
	 */
	public function Eliminar() {
		Utils::EliminarElemento(ELEMENTOS_OPERADOR, $this->IdOperador);
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdOperador = $this->Nombre = $this->Domicilio = $this->Localidad = $this->Telefono = $this->Email = $this->Contacto = $this->Usuario = $this->Clave = $this->Comentario = $this->ControlHistorial = $this->Vencimientos = $this->Servicios = $this->Proyectosencola = $this->Habilitado = null;
	}
	
	public static function Permiso($nIdOperador) {
		$oDB = new Dbconnection();
		
		$cSql = "SELECT * FROM operadores_permisos WHERE id_operador = '{$nIdOperador}' LIMIT 1";
		$oDB->Query($cSql);
		$aPermisos = $oDB->Field();
		unset($aPermisos['id_operador']);
		return array_search(1, $aPermisos);
	}
}
?>