<?
class Operadores_fp extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdOperador, $IdProyecto, $IdFase, $Posicion;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdOperador !== null) {
			$aWhere[] = "operadores_fp.id_operador = '{$this->IdOperador}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "operadores_fp.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdFase !== null) {
			$aWhere[] = "operadores_fp.id_fase = '{$this->IdFase}'";
		}
		if (@$this->IdFaseactiva !== null) {
			if ($this->IdFaseactiva === false) {
				$aWhere[] = "proyectos.id_faseactiva IS NULL";
			}
			else {
				$aWhere[] = "proyectos.id_faseactiva = '{$this->IdFaseactiva}'";
			}
		}
		if ($this->Posicion !== null) {
			$aWhere[] = "operadores_fp.posicion = '{$this->Posicion}'";
		}
		if (@$this->IdEstadofase !== null) {
			$aWhere[] = "proyectos.id_estadofase = '{$this->IdEstadofase}'";
		}
		if (@$this->SinCerrar === true) {
			$aWhere[] = "proyectos.fin_real IS NULL";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSelect = @$this->Distintos ? 'DISTINCT proyectos.*,' : 'proyectos.*, operadores_fp.*,';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS {$cSelect} operadores.nombre operador, clientes.nombre cliente, tipos_proyecto.nombre tipo_proyecto
			FROM operadores_fp 
			INNER JOIN proyectos ON proyectos.id_proyecto = operadores_fp.id_proyecto
			LEFT JOIN operadores ON operadores.id_operador = operadores_fp.id_operador
			LEFT JOIN clientes ON clientes.id_cliente = proyectos.id_cliente
			LEFT JOIN tipos_proyecto ON tipos_proyecto.id_tipoproyecto = proyectos.id_tipoproyecto
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function BuscarPendientes() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdOperador !== null) {
			$aWhere[] = "operadores_fp.id_operador = '{$this->IdOperador}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "operadores_fp.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdFase !== null) {
			$aWhere[] = "operadores_fp.id_fase = '{$this->IdFase}'";
		}
		if ($this->Posicion !== null) {
			$aWhere[] = "operadores_fp.posicion = '{$this->Posicion}'";
		}
		
		$aWhere[] = "fases_proyecto.inicio_estimado <= NOW() AND fases_proyecto.inicio_real IS NULL";
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS operadores_fp.*, operadores.nombre operador, proyectos.nombre proyecto, clientes.nombre cliente
			FROM operadores_fp 
			LEFT JOIN fases_proyecto ON fases_proyecto.id_proyecto = operadores_fp.id_proyecto AND fases_proyecto.id_fase = operadores_fp.id_fase
			LEFT JOIN operadores ON operadores.id_operador = operadores_fp.id_operador
			LEFT JOIN proyectos ON proyectos.id_proyecto = operadores_fp.id_proyecto
			INNER JOIN clientes ON clientes.id_cliente = proyectos.id_cliente
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function BuscarActivos() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdOperador !== null) {
			$aWhere[] = "operadores_fp.id_operador = '{$this->IdOperador}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "operadores_fp.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdFase !== null) {
			$aWhere[] = "operadores_fp.id_fase = '{$this->IdFase}'";
		}
		if ($this->Posicion !== null) {
			$aWhere[] = "operadores_fp.posicion = '{$this->Posicion}'";
		}
		
		// Esto queda comentado, sino, no muestra el operador cuando no esta completa la fecha de la fase.
		//$aWhere[] = "fases_proyecto.inicio_real <= NOW() AND fases_proyecto.fin_real IS NULL";
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS operadores_fp.*, operadores.nombre operador, proyectos.nombre proyecto, clientes.nombre cliente, tipos_proyecto.nombre tipo_proyecto
			FROM operadores_fp 
			LEFT JOIN fases_proyecto ON fases_proyecto.id_proyecto = operadores_fp.id_proyecto AND fases_proyecto.id_fase = operadores_fp.id_fase
			LEFT JOIN operadores ON operadores.id_operador = operadores_fp.id_operador
			LEFT JOIN proyectos ON proyectos.id_proyecto = operadores_fp.id_proyecto
			INNER JOIN clientes ON clientes.id_cliente = proyectos.id_cliente
			LEFT JOIN tipos_proyecto ON tipos_proyecto.id_tipoproyecto = proyectos.id_tipoproyecto
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function BuscarOperadores() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdProyecto !== null) {
			//$aWhere[] = "operadores_fp.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdFase !== null) {
			$aWhere[] = "fases_operador.id_fase = '{$this->IdFase}'";
		}
		if ($this->Posicion !== null) {
			$aWhere[] = "fases_operador.posicion = '{$this->Posicion}'";
		}
		// Solo operadores habilitados
		$aWhere[] = "operadores.habilitado = 1";
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		/*
		echo $cSql = "
			SELECT SQL_CALC_FOUND_ROWS fases_operador.*, operadores.nombre operador, operadores_fp.id_proyecto asignado
			FROM fases_operador
			LEFT JOIN operadores ON operadores.id_operador = fases_operador.id_operador
			LEFT JOIN operadores_fp ON operadores_fp.id_proyecto = '{$this->IdProyecto}' AND operadores_fp.id_fase = '{$this->IdFase}'
			{$cWhere} {$cOrderBy} {$cLimit}";*/
		
		
		$cSql = "
			SELECT SQL_CALC_FOUND_ROWS fases_operador.*, operadores.nombre operador, operadores_fp.id_proyecto asignado
			FROM operadores
			INNER JOIN fases_operador ON fases_operador.id_operador = operadores.id_operador
			LEFT JOIN operadores_fp ON operadores_fp.id_operador = operadores.id_operador AND operadores_fp.id_fase = fases_operador.id_fase AND operadores_fp.id_proyecto = '{$this->IdProyecto}'
			{$cWhere} {$cOrderBy} {$cLimit}";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT operadores_fp.* FROM operadores_fp WHERE operadores_fp.id_operador = '{$this->IdOperador}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$cSql = "REPLACE INTO operadores_fp SET ".
			"id_proyecto = '{$this->IdProyecto}', ".
			"id_fase = '{$this->IdFase}', ".
			"id_operador = '{$this->IdOperador}', ".
			"posicion = '{$this->Posicion}'";
		
		$this->DB->Query($cSql);
		
		return true;
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		
		if ($this->IdOperador !== null) {
			$cSql = "DELETE IGNORE FROM operadores_fp WHERE id_proyecto = '{$this->IdProyecto}' AND id_fase = '{$this->IdFase}' AND id_operador = '{$this->IdOperador}' LIMIT 1";
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "DELETE IGNORE FROM operadores_fp WHERE id_proyecto = '{$this->IdProyecto}' AND id_fase = '{$this->IdFase}'";
			$this->DB->Query($cSql);
		}
		
		return $this->DB->AffectedRows();
	}
	
	public function UltimaPosicion() {
		$cSql = "SELECT MAX(operadores_fp.posicion) maximo FROM operadores_fp INNER JOIN proyectos ON proyectos.id_proyecto = operadores_fp.id_proyecto AND proyectos.id_estadofase = 1 WHERE id_operador = '{$this->IdOperador}' AND id_fase = '{$this->IdFase}'";
		$this->DB->Query($cSql);
		
		if ($this->DB->Count()) {
			return $this->DB->Field('maximo') + 1;
		}
		else {
			return 0;
		}
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdOperador = $aDatos['id_operador'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->IdFase = $aDatos['id_fase'];
		$this->Posicion = $aDatos['posicion'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdOperador = $this->IdProyecto = $this->IdFase = $this->Posicion = null;
	}
}
?>