<?
class Ordenes extends Model {
	
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdOrden, $Fecha, $IdCliente, $Descripcion, $DescripcionFactura, $Importe, $IdFactura;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Ordenes.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cSelect = $cGroupBy = $cOrderBy = $cLimit = '';
		$aWhere = array();
		
		if (@$this->GroupBy) {
			$cSelect = $this->GroupBy['select'].', ';
			$cGroupBy = 'GROUP BY '.$this->GroupBy['group'];
		}
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Fecha !== null) {
			$aWhere[] = "ordenes.fecha = '{$this->Fecha}'";
		}
		if (@$this->Desde !== null) {
			$aWhere[] = "ordenes.fecha >= '{$this->Desde}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "ordenes.fecha <= '{$this->Hasta}'";
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "ordenes.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->Descripcion !== null) {
			$aWhere[] = "ordenes.descripcion = '{$this->Descripcion}'";
		}
		if ($this->Importe !== null) {
			$aWhere[] = "ordenes.importe = '{$this->Importe}'";
		}
		if ($this->IdFactura !== null) {
			$aWhere[] = "ordenes.id_factura = '{$this->IdFactura}'";
		}
		if (@$this->Facturado !== null) {
			if ($this->Facturado == true) {
				$aWhere[] = "ordenes.id_factura IS NOT NULL";
			}
			else {
				$aWhere[] = "ordenes.id_factura IS NULL";
			}
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS {$cSelect} ordenes.*, clientes.razon_social, clientes.nombre cliente
			FROM ordenes 
			LEFT JOIN clientes ON clientes.id_cliente = ordenes.id_cliente 
		{$cWhere} {$cGroupBy} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	public function BuscarDeudas() {
		$cSql = "
			SELECT clientes.*, SUM(ordenes.importe) total
			FROM ordenes 
			LEFT JOIN clientes ON clientes.id_cliente = ordenes.id_cliente
			WHERE ordenes.id_factura IS NULL AND ordenes.fecha <= NOW()
			GROUP BY ordenes.id_cliente
			ORDER BY clientes.nombre
			";
		$this->DB->Query($cSql);
		
		return $this->DB->GetRecordset();
	}
	
	/**
	 * @desc
	 * Devuelve el Ordenes con ID $this->IdOrden.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT ordenes.*, clientes.razon_social, clientes.nombre cliente
			FROM ordenes 
			LEFT JOIN clientes ON clientes.id_cliente = ordenes.id_cliente
			WHERE ordenes.id_orden = '{$this->IdOrden}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Ordenes.
	 * Devuelve el ID del Ordenes guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		if ($this->IdOrden) {
			$cSql = "UPDATE ordenes SET ".
				"fecha = '{$this->Fecha}', ".
				"id_cliente = '{$this->IdCliente}', ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"descripcion_factura = ".($this->DescripcionFactura ? "'{$this->DescripcionFactura}'" : 'NULL').", ".
				"importe = '{$this->Importe}', ".
				"id_factura = ".($this->IdFactura ? "'{$this->IdFactura}'" : 'NULL')." ".
				"WHERE id_orden = '{$this->IdOrden}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO ordenes SET ".
				"fecha = '{$this->Fecha}', ".
				"id_cliente = '{$this->IdCliente}', ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"descripcion_factura = ".($this->DescripcionFactura ? "'{$this->DescripcionFactura}'" : 'NULL').", ".
				"importe = '{$this->Importe}', ".
				"id_factura = ".($this->IdFactura ? "'{$this->IdFactura}'" : 'NULL')."";
			
			$this->IdOrden = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			return 0;
		}
		else {
			return $this->IdOrden;
		}
	}
	
	public function Saldo() {
		if ($this->IdCliente) {
			$cSql = "SELECT SUM(ordenes.importe) total 
				FROM ordenes 
				WHERE ordenes.id_cliente = '{$this->IdCliente}' AND ordenes.id_factura IS NULL AND ordenes.fecha <= CURRENT_DATE()";
		}
		elseif (@$this->Desde) {
			$cSql = "SELECT SUM(ordenes.importe) total 
				FROM ordenes 
				WHERE ordenes.fecha >= '{$this->Desde}' AND ordenes.fecha <= '{$this->Hasta}'";
		}
		$this->DB->Query($cSql);
		
		return $this->DB->Field('total');
	}
	
	/**
	 * @desc
	 * Devuelve un array con la cantidad de ordenes y saldo por facturar.
	 *
	 * @return array
	 */
	public function SaldoPorFacturar($nReVencidos=false) {
		
		$cSql = "SELECT COUNT(ordenes.id_orden) cantidad, SUM(ordenes.importe) importe
		FROM ordenes
		/*WHERE ordenes.id_factura IS NULL AND ordenes.fecha <= DATE_SUB(CURRENT_DATE(), INTERVAL 45 DAY)*/
		WHERE ordenes.id_factura IS NULL AND ordenes.fecha <= CURRENT_DATE()
		";
		
		$this->DB->Query($cSql);
		
		return $this->DB->Field();
	}
	
	/**
	 * @desc
	 * Elimina un Ordenes.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE FROM ordenes WHERE id_orden = '{$this->IdOrden}' LIMIT 1";
		$this->DB->Query($cSql);
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdOrden = $aDatos['id_orden'];
		$this->Fecha = $aDatos['fecha'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->Descripcion = $aDatos['descripcion'];
		$this->DescripcionFactura = $aDatos['descripcion_factura'];
		$this->Importe = $aDatos['importe'];
		$this->IdFactura = $aDatos['id_factura'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdOrden = $this->Fecha = $this->IdCliente = $this->Descripcion = $this->DescripcionFactura = $this->Importe = $this->IdFactura = null;
	}
}
?>