<?
class Preclientes extends Model {
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdPrecliente, $FechaAlta, $IdComercial, $RazonSocial, $IdCondicionfiscal, $Cuit, $Domicilio, $Localidad, $Telefono, $Email, $Web, $IdRubro, $Observacion;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->FechaAlta !== null) {
			$aWhere[] = "preclientes.fecha_alta = '{$this->FechaAlta}'";
		}
		if ($this->IdComercial !== null) {
			$aWhere[] = "preclientes.id_comercial = '{$this->IdComercial}'";
		}
		if ($this->RazonSocial !== null) {
			$aWhere[] = "preclientes.razon_social LIKE '{$this->RazonSocial}%'";
		}
		if ($this->IdCondicionfiscal !== null) {
			$aWhere[] = "preclientes.id_condicionfiscal = '{$this->IdCondicionfiscal}'";
		}
		if ($this->Cuit !== null) {
			$aWhere[] = "preclientes.cuit = '{$this->Cuit}'";
		}
		if ($this->Domicilio !== null) {
			$aWhere[] = "preclientes.domicilio = '{$this->Domicilio}'";
		}
		if ($this->Localidad !== null) {
			$aWhere[] = "preclientes.localidad = '{$this->Localidad}'";
		}
		if ($this->Telefono !== null) {
			$aWhere[] = "preclientes.telefono = '{$this->Telefono}'";
		}
		if ($this->Email !== null) {
			$aWhere[] = "preclientes.email = '{$this->Email}'";
		}
		if ($this->Web !== null) {
			$aWhere[] = "preclientes.web = '{$this->Web}'";
		}
		if ($this->IdRubro !== null) {
			$aWhere[] = "preclientes.id_rubro = '{$this->IdRubro}'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "preclientes.razon_social REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "preclientes.razon_social LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS preclientes.* FROM preclientes {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT preclientes.*, condiciones_fiscales.nombre condicion_fiscal
			FROM preclientes 
			LEFT JOIN condiciones_fiscales ON condiciones_fiscales.id_condicionfiscal = preclientes.id_condicionfiscal
			WHERE preclientes.id_precliente = '{$this->IdPrecliente}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdPrecliente) {
			$cSql = "UPDATE preclientes SET ".
				"fecha_alta = ".($this->FechaAlta ? "'{$this->FechaAlta}'" : 'NULL').", ".
				"id_comercial = '{$this->IdComercial}', ".
				"razon_social = '{$this->RazonSocial}', ".
				"id_condicionfiscal = ".($this->IdCondicionfiscal ? "'{$this->IdCondicionfiscal}'" : 'NULL').", ".
				"cuit = ".($this->Cuit ? "'{$this->Cuit}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"localidad = ".($this->Localidad ? "'{$this->Localidad}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"web = ".($this->Web ? "'{$this->Web}'" : 'NULL').", ".
				"id_rubro = ".($this->IdRubro ? "'{$this->IdRubro}'" : 'NULL').", ".
				"observacion = ".($this->Observacion ? "'{$this->Observacion}'" : 'NULL')." ".
				"WHERE id_precliente = '{$this->IdPrecliente}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO preclientes SET ".
				"fecha_alta = ".($this->FechaAlta ? "'{$this->FechaAlta}'" : 'NULL').", ".
				"id_comercial = '{$this->IdComercial}', ".
				"razon_social = '{$this->RazonSocial}', ".
				"id_condicionfiscal = ".($this->IdCondicionfiscal ? "'{$this->IdCondicionfiscal}'" : 'NULL').", ".
				"cuit = ".($this->Cuit ? "'{$this->Cuit}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"localidad = ".($this->Localidad ? "'{$this->Localidad}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"web = ".($this->Web ? "'{$this->Web}'" : 'NULL').", ".
				"id_rubro = ".($this->IdRubro ? "'{$this->IdRubro}'" : 'NULL').", ".
				"observacion = ".($this->Observacion ? "'{$this->Observacion}'" : 'NULL');
			
			$this->IdPrecliente = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdPrecliente;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM preclientes WHERE id_precliente = '{$this->IdPrecliente}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdPrecliente = $aDatos['id_precliente'];
		$this->FechaAlta = $aDatos['fecha_alta'];
		$this->IdComercial = $aDatos['id_comercial'];
		$this->RazonSocial = $aDatos['razon_social'];
		$this->IdCondicionfiscal = $aDatos['id_condicionfiscal'];
		$this->Cuit = $aDatos['cuit'];
		$this->Domicilio = $aDatos['domicilio'];
		$this->Localidad = $aDatos['localidad'];
		$this->Telefono = $aDatos['telefono'];
		$this->Email = $aDatos['email'];
		$this->Web = $aDatos['web'];
		$this->IdRubro = $aDatos['id_rubro'];
		$this->Observacion = $aDatos['observacion'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdPrecliente = $this->FechaAlta = $this->IdComercial = $this->RazonSocial = $this->IdCondicionfiscal = $this->Cuit = $this->Domicilio = $this->Localidad = $this->Telefono = $this->Email = $this->Web = $this->IdRubro = $this->Observacion = null;
	}
}
?>