<?
class Proveedores extends Model {
	
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdProveedor, $Denominacion, $RazonSocial, $Cuit, $Email, $Telefono, $Domicilio, $PaginaWeb, $Usuario, $Clave, $Comentarios;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Proveedores.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Denominacion !== null) {
			$aWhere[] = "proveedores.denominacion LIKE '%{$this->Denominacion}%'";
		}
		if ($this->RazonSocial !== null) {
			$aWhere[] = "proveedores.razon_social = '{$this->RazonSocial}'";
		}
		if ($this->Cuit !== null) {
			$aWhere[] = "proveedores.cuit = '{$this->Cuit}'";
		}
		if ($this->Email !== null) {
			$aWhere[] = "proveedores.email = '{$this->Email}'";
		}
		if ($this->Telefono !== null) {
			$aWhere[] = "proveedores.telefono = '{$this->Telefono}'";
		}
		if ($this->Domicilio !== null) {
			$aWhere[] = "proveedores.domicilio = '{$this->Domicilio}'";
		}
		if ($this->PaginaWeb !== null) {
			$aWhere[] = "proveedores.pagina_web = '{$this->PaginaWeb}'";
		}
		if ($this->Usuario !== null) {
			$aWhere[] = "proveedores.usuario = '{$this->Usuario}'";
		}
		if ($this->Clave !== null) {
			$aWhere[] = "proveedores.clave = '{$this->Clave}'";
		}
		if ($this->Comentarios !== null) {
			$aWhere[] = "proveedores.comentarios = '{$this->Comentarios}'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "proveedores.denominacion REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "proveedores.denominacion LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS proveedores.* FROM proveedores INNER JOIN elementos ON elementos.id_entidad = proveedores.id_proveedor AND elementos.id_tipoelemento = ".ELEMENTOS_PROVEEDOR." AND elementos.eliminado = 0 {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Proveedores con ID $this->IdProveedor.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT proveedores.* FROM proveedores WHERE proveedores.id_proveedor = '{$this->IdProveedor}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Proveedores.
	 * Devuelve el ID del Proveedores guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdProveedor) {
			$cSql = "UPDATE proveedores SET ".
				"denominacion = '{$this->Denominacion}', ".
				"razon_social = ".($this->RazonSocial ? "'{$this->RazonSocial}'" : 'NULL').", ".
				"cuit = ".($this->Cuit ? "'{$this->Cuit}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"pagina_web = ".($this->PaginaWeb ? "'{$this->PaginaWeb}'" : 'NULL').", ".
				"usuario = ".($this->Usuario ? "'{$this->Usuario}'" : 'NULL').", ".
				"clave = ".($this->Clave ? "'{$this->Clave}'" : 'NULL').", ".
				"comentarios = ".($this->Comentarios ? "'{$this->Comentarios}'" : 'NULL')." ".
				"WHERE id_proveedor = '{$this->IdProveedor}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO proveedores SET ".
				"denominacion = '{$this->Denominacion}', ".
				"razon_social = ".($this->RazonSocial ? "'{$this->RazonSocial}'" : 'NULL').", ".
				"cuit = ".($this->Cuit ? "'{$this->Cuit}'" : 'NULL').", ".
				"email = ".($this->Email ? "'{$this->Email}'" : 'NULL').", ".
				"telefono = ".($this->Telefono ? "'{$this->Telefono}'" : 'NULL').", ".
				"domicilio = ".($this->Domicilio ? "'{$this->Domicilio}'" : 'NULL').", ".
				"pagina_web = ".($this->PaginaWeb ? "'{$this->PaginaWeb}'" : 'NULL').", ".
				"usuario = ".($this->Usuario ? "'{$this->Usuario}'" : 'NULL').", ".
				"clave = ".($this->Clave ? "'{$this->Clave}'" : 'NULL').", ".
				"comentarios = ".($this->Comentarios ? "'{$this->Comentarios}'" : 'NULL')."";
			
			$this->IdProveedor = $this->DB->QueryInsert($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_PROVEEDOR, $this->IdProveedor);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdProveedor;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Proveedores.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		Utils::EliminarElemento(ELEMENTOS_PROVEEDOR, $this->IdProveedor);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdProveedor = $aDatos['id_proveedor'];
		$this->Denominacion = $aDatos['denominacion'];
		$this->RazonSocial = $aDatos['razon_social'];
		$this->Cuit = $aDatos['cuit'];
		$this->Email = $aDatos['email'];
		$this->Telefono = $aDatos['telefono'];
		$this->Domicilio = $aDatos['domicilio'];
		$this->PaginaWeb = $aDatos['pagina_web'];
		$this->Usuario = $aDatos['usuario'];
		$this->Clave = $aDatos['clave'];
		$this->Comentarios = $aDatos['comentarios'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdProveedor = $this->Denominacion = $this->RazonSocial = $this->Cuit = $this->Email = $this->Telefono = $this->Domicilio = $this->PaginaWeb = $this->Usuario = $this->Clave = $this->Comentarios = null;
	}
}
?>