<?
class Proyectos extends Model {
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdProyecto, $Orden, $IdCliente, $IdTipoproyecto, $IdFaseactiva, $IdEstadofase, $Posicion, $FechaFase, $Nombre, $FechaCarga, $InicioEstimado, $FinEstimado, $InicioReal, $FinReal, $Descripcion, $Comentario, $NotasPrivadas;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdCliente !== null) {
			$aWhere[] = "proyectos.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->Orden !== null) {
			$aWhere[] = "proyectos.orden = '{$this->Orden}'";
		}
		if ($this->IdTipoproyecto !== null) {
			$aWhere[] = "proyectos.id_tipoproyecto = '{$this->IdTipoproyecto}'";
		}
		if ($this->IdFaseactiva !== null) {
			if ($this->IdFaseactiva === false) {
				$aWhere[] = "proyectos.id_faseactiva IS NULL";
			}
			else {
				$aWhere[] = "proyectos.id_faseactiva = '{$this->IdFaseactiva}'";
			}
		}
		if ($this->IdEstadofase !== null) {
			$aWhere[] = "proyectos.id_estadofase = '{$this->IdEstadofase}'";
		}
		if ($this->Posicion !== null) {
			$aWhere[] = "proyectos.posicion = '{$this->Posicion}'";
		}
		if ($this->FechaCarga !== null) {
			$aWhere[] = "proyectos.fecha_fase = '{$this->FechaFase}'";
		}
		if ($this->Nombre !== null) {
			$aWhere[] = "proyectos.nombre LIKE '%{$this->Nombre}%'";
		}
		if ($this->FechaCarga !== null) {
			$aWhere[] = "proyectos.fecha_carga = '{$this->FechaCarga}'";
		}
		if ($this->InicioEstimado !== null) {
			$aWhere[] = "proyectos.inicio_estimado = '{$this->InicioEstimado}'";
		}
		if ($this->FinEstimado !== null) {
			$aWhere[] = "proyectos.fin_estimado = '{$this->FinEstimado}'";
		}
		if ($this->InicioReal !== null) {
			if ($this->InicioReal === true) {
				$aWhere[] = "proyectos.inicio_real IS NOT NULL";
			}
			elseif ($this->InicioReal === false) {
				$aWhere[] = "proyectos.inicio_real IS NULL";
			}
			else {
				$aWhere[] = "proyectos.inicio_real = '{$this->InicioReal}'";
			}
		}
		if ($this->FinReal !== null) {
			if ($this->FinReal === true) {
				$aWhere[] = "proyectos.fin_real IS NOT NULL";
			}
			elseif ($this->FinReal === false) {
				$aWhere[] = "proyectos.fin_real IS NULL";
			}
			else {
				$aWhere[] = "proyectos.fin_real = '{$this->FinReal}'";
			}
		}
		if (isset($this->FiltroAbiertos)) {
			$aWhere[] = "proyectos.inicio_real <= '{$this->FiltroAbiertos}'";
		}
		if ($this->Descripcion !== null) {
			$aWhere[] = "proyectos.descripcion = '{$this->Descripcion}'";
		}
		if ($this->NotasPrivadas !== null) {
			$aWhere[] = "proyectos.notas_privadas = '{$this->NotasPrivadas}'";
		}
		if (@$this->FinalizadosAl !== null) {
			$aWhere[] = "proyectos.fin_estimado <= '{$this->FinalizadosAl}'";
		}
		if (@$this->EstimadosAl !== null) {
			$aWhere[] = "proyectos.inicio_estimado <= '{$this->EstimadosAl}'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "clientes.nombre REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "clientes.nombre LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS proyectos.*, clientes.nombre cliente, tipos_proyecto.nombre tipo_proyecto
			FROM proyectos 
			LEFT JOIN clientes ON clientes.id_cliente = proyectos.id_cliente
			LEFT JOIN tipos_proyecto ON tipos_proyecto.id_tipoproyecto = proyectos.id_tipoproyecto
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT proyectos.*, clientes.nombre cliente, tipos_proyecto.nombre tipo_proyecto
			FROM proyectos 
			LEFT JOIN clientes ON clientes.id_cliente = proyectos.id_cliente
			LEFT JOIN tipos_proyecto ON tipos_proyecto.id_tipoproyecto = proyectos.id_tipoproyecto
			WHERE proyectos.id_proyecto = '{$this->IdProyecto}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdProyecto) {
			$cSql = "UPDATE proyectos SET ".
				"id_cliente = '{$this->IdCliente}', ".
				"orden = ".($this->Orden ? "'{$this->Orden}'" : 'NULL').", ".
				"id_tipoproyecto = '{$this->IdTipoproyecto}', ".
				"id_faseactiva = '{$this->IdFaseactiva}', ".
				"id_estadofase = '{$this->IdEstadofase}', ".
				"posicion = '{$this->Posicion}', ".
				"fecha_fase = ".($this->FechaFase ? "'{$this->FechaFase}'" : 'NULL').", ".
				"nombre = '{$this->Nombre}', ".
				"fecha_carga = '{$this->FechaCarga}', ".
				"inicio_estimado = '{$this->InicioEstimado}', ".
				"fin_estimado = '{$this->FinEstimado}', ".
				"inicio_real = ".($this->InicioReal ? "'{$this->InicioReal}'" : 'NULL').", ".
				"fin_real = ".($this->FinReal ? "'{$this->FinReal}'" : 'NULL').", ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL').", ".
				"notas_privadas = ".($this->NotasPrivadas ? "'{$this->NotasPrivadas}'" : 'NULL')." ".
				"WHERE id_proyecto = '{$this->IdProyecto}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO proyectos SET ".
				"id_cliente = '{$this->IdCliente}', ".
				"orden = ".($this->Orden ? "'{$this->Orden}'" : 'NULL').", ".
				"id_tipoproyecto = '{$this->IdTipoproyecto}', ".
				"id_faseactiva = '{$this->IdFaseactiva}', ".
				"id_estadofase = '{$this->IdEstadofase}', ".
				"posicion = '{$this->Posicion}', ".
				"fecha_fase = ".($this->FechaFase ? "'{$this->FechaFase}'" : 'NULL').", ".
				"nombre = '{$this->Nombre}', ".
				"fecha_carga = '{$this->FechaCarga}', ".
				"inicio_estimado = '{$this->InicioEstimado}', ".
				"fin_estimado = '{$this->FinEstimado}', ".
				"inicio_real = ".($this->InicioReal ? "'{$this->InicioReal}'" : 'NULL').", ".
				"fin_real = ".($this->FinReal ? "'{$this->FinReal}'" : 'NULL').", ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL').", ".
				"notas_privadas = ".($this->NotasPrivadas ? "'{$this->NotasPrivadas}'" : 'NULL')."";
			
			$this->IdProyecto = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdProyecto;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE FROM proyectos WHERE id_proyecto = '{$this->IdProyecto}'";
		$this->DB->Query($cSql);
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->Orden = $aDatos['orden'];
		$this->IdTipoproyecto = $aDatos['id_tipoproyecto'];
		$this->IdFaseactiva = $aDatos['id_faseactiva'];
		$this->IdEstadofase = $aDatos['id_estadofase'];
		$this->Posicion = $aDatos['posicion'];
		$this->FechaFase = $aDatos['fecha_fase'];
		$this->Nombre = $aDatos['nombre'];
		$this->FechaCarga = $aDatos['fecha_carga'];
		$this->InicioEstimado = $aDatos['inicio_estimado'];
		$this->FinEstimado = $aDatos['fin_estimado'];
		$this->InicioReal = $aDatos['inicio_real'];
		$this->FinReal = $aDatos['fin_real'];
		$this->Descripcion = $aDatos['descripcion'];
		$this->Comentario = $aDatos['comentario'];
		$this->NotasPrivadas = $aDatos['notas_privadas'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdProyecto = $this->Orden  = $this->IdCliente= $this->IdTipoproyecto = $this->IdFaseactiva = $this->IdEstadofase = $this->Posicion = $this->FechaFase = $this->Nombre = $this->FechaCarga = $this->InicioEstimado = $this->FinEstimado = $this->InicioReal = $this->FinReal = $this->Descripcion = $this->Comentario = $this->NotasPrivadas = null;
	}
	
	public static function ObtenerUltimaPosicion($nIdFase, $nIdEstado) {
		$oDB = new Dbconnection();
		
		$cSql = "SELECT MAX(posicion) posicion FROM proyectos WHERE id_faseactiva = '{$nIdFase}' AND id_estadofase = '{$nIdEstado}'";
		$oDB->Query($cSql);
		
		return $oDB->Field('posicion') + 1;
	}
}
?>