<?
class Seguimientos extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdSeguimiento, $IdSector, $IdEntidad, $IdOperador, $Fecha, $Texto;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdSector !== null) {
			$aWhere[] = "seguimientos.id_sector = '{$this->IdSector}'";
		}
		if ($this->IdEntidad !== null) {
			$aWhere[] = "seguimientos.id_entidad = '{$this->IdEntidad}'";
		}
		if ($this->IdOperador !== null) {
			$aWhere[] = "seguimientos.id_operador = '{$this->IdOperador}'";
		}
		if ($this->Fecha !== null) {
			$aWhere[] = "seguimientos.fecha = '{$this->Fecha}'";
		}
		if ($this->Texto !== null) {
			$aWhere[] = "seguimientos.texto = '{$this->Texto}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS seguimientos.*, GROUP_CONCAT(CONCAT(flags_seguimiento.valor, ',')) flags, operadores.nombre operador
			FROM seguimientos 
			LEFT JOIN flags_seguimiento ON flags_seguimiento.id_seguimiento = seguimientos.id_seguimiento
			LEFT JOIN operadores ON operadores.id_operador = seguimientos.id_operador
			{$cWhere} GROUP BY seguimientos.id_seguimiento {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		
		//echo $cSql; die;
		
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT seguimientos.* FROM seguimientos WHERE seguimientos.id_seguimiento = '{$this->IdSeguimiento}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
    /**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function BuscarTickets() {
		$cSql = "SELECT flags_seguimiento.valor estado, seguimientos.fecha, seguimientos.id_operador, proyectos.id_cliente, proyectos.nombre proyecto, clientes.nombre, proyectos.id_proyecto".
				" FROM ((proyectos inner join seguimientos on seguimientos.id_entidad = proyectos.id_proyecto)".
				" LEFT JOIN clientes on clientes.id_cliente = proyectos.id_cliente)".
				" LEFT JOIN flags_seguimiento on seguimientos.id_seguimiento = flags_seguimiento.id_seguimiento".
				" ORDER BY seguimientos.id_seguimiento DESC LIMIT 1500";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		
		if (!$this->IdOperador) {
			$this->IdOperador = $_SESSION['operador']['id_operador'];
		}
		
		$this->DB->Begin();
		
		if ($this->IdSeguimiento) {
			$cSql = "UPDATE seguimientos SET ".
				"id_sector = '{$this->IdSector}', ".
				"id_entidad = '{$this->IdEntidad}', ".
				"id_operador = '{$this->IdOperador}', ".
				"fecha = '{$this->Fecha}', ".
				"texto = '{$this->Texto}' ".
				"WHERE id_seguimiento = '{$this->IdSeguimiento}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO seguimientos SET ".
				"id_sector = '{$this->IdSector}', ".
				"id_entidad = '{$this->IdEntidad}', ".
				"id_operador = '{$this->IdOperador}', ".
				"fecha = '{$this->Fecha}', ".
				"texto = '{$this->Texto}'";
			
			$this->IdSeguimiento = $this->DB->QueryInsert($cSql);
			
			//Utils::GuardarElemento(ELEMENTOS_SEGUIMIENTO, $this->IdSeguimiento);
			
			foreach ($this->Flags as $item) {
				$cSql = "INSERT IGNORE INTO flags_seguimiento SET id_seguimiento = '{$this->IdSeguimiento}', valor = '{$item}'";
				$this->DB->Query($cSql);
			}
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdSeguimiento;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE FROM seguimientos WHERE id_seguimiento = '{$this->IdSeguimiento}' LIMIT 1";
		$this->DB->Query($cSql);
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdSeguimiento = $aDatos['id_seguimiento'];
		$this->IdSector = $aDatos['id_sector'];
		$this->IdEntidad = $aDatos['id_entidad'];
		$this->IdOperador = $aDatos['id_operador'];
		$this->Fecha = $aDatos['fecha'];
		$this->Texto = $aDatos['texto'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdSeguimiento = $this->IdSector = $this->IdEntidad = $this->IdOperador = $this->Fecha = $this->Texto = null;
	}
}
?>