<?
class Seguimientos_fase extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdSeguimientofase, $IdProyecto, $IdFase, $IdEstadofase, $Fecha;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdProyecto !== null) {
			$aWhere[] = "seguimientos_fase.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->IdFase !== null) {
			$aWhere[] = "seguimientos_fase.id_fase = '{$this->IdFase}'";
		}
		if ($this->IdEstadofase !== null) {
			$aWhere[] = "seguimientos_fase.id_estadofase = '{$this->IdEstadofase}'";
		}
		if ($this->Fecha !== null) {
			$aWhere[] = "seguimientos_fase.fecha = '{$this->Fecha}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS seguimientos_fase.*, fases.nombre fase
			FROM seguimientos_fase 
			LEFT JOIN fases ON fases.id_fase = seguimientos_fase.id_fase
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT seguimientos_fase.* FROM seguimientos_fase WHERE seguimientos_fase.id_seguimientofase = '{$this->IdSeguimientofase}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdSeguimientofase) {
			$cSql = "UPDATE seguimientos_fase SET ".
				"id_proyecto = '{$this->IdProyecto}', ".
				"id_fase = '{$this->IdFase}', ".
				"id_estadofase = '{$this->IdEstadofase}', ".
				"fecha = ".($this->Fecha ? "'{$this->Fecha}'" : 'NULL')." ".
				"WHERE id_seguimientofase = '{$this->IdSeguimientofase}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO seguimientos_fase SET ".
				"id_proyecto = '{$this->IdProyecto}', ".
				"id_fase = '{$this->IdFase}', ".
				"id_estadofase = '{$this->IdEstadofase}' ";
			
			$this->IdSeguimientofase = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdSeguimientofase;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM seguimientos_fase WHERE id_seguimientofase = '{$this->IdSeguimientofase}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdSeguimientofase = $aDatos['id_seguimientofase'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->IdFase = $aDatos['id_fase'];
		$this->IdEstadofase = $aDatos['id_estadofase'];
		$this->Fecha = $aDatos['fecha'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdSeguimientofase = $this->IdProyecto = $this->IdFase = $this->IdEstadofase = $this->Fecha = null;
	}
}
?>