<?
class Servicios extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdServicio, $IdTiposervicio, $Horas, $IdCliente, $IdProyecto, $Nombre, $FechaAlta, $FechaActivacion, $FechaVencimiento, $FechaFacturacion, $FechaBaja, $TextoOrden, $IdPeriodo, $Importe, $Cuotas, $Descripcion, $Comentario, $IdOrden;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdTiposervicio !== null) {
			$aWhere[] = "servicios.id_tiposervicio = '{$this->IdTiposervicio}'";
		}
		if ($this->Horas !== null) {
			if ($this->Horas === true) {
				$aWhere[] = "servicios.horas IS NOT NULL";
			}
			elseif ($this->Horas === false) {
				$aWhere[] = "servicios.horas IS NULL";
			}
			else {
				$aWhere[] = "servicios.horas = '{$this->Horas}'";
			}
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "servicios.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "servicios.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->Nombre !== null) {
			$aWhere[] = "servicios.nombre = '{$this->Nombre}'";
		}
		if ($this->FechaAlta !== null) {
			$aWhere[] = "servicios.fecha_alta = '{$this->FechaAlta}'";
		}
		if ($this->FechaActivacion !== null) {
			$aWhere[] = "servicios.fecha_activacion = '{$this->FechaActivacion}'";
		}
		if ($this->FechaBaja !== null) {
			if ($this->FechaBaja === true) {
				$aWhere[] = "servicios.fecha_baja IS NOT NULL";
			}
			elseif ($this->FechaBaja === false) {
				$aWhere[] = "servicios.fecha_baja IS NULL";
			}
			else {
				$aWhere[] = "servicios.fecha_baja = '{$this->FechaBaja}'";
			}
		}
		if ($this->FechaVencimiento !== null) {
			if ($this->FechaVencimiento === true) {
				$aWhere[] = "servicios.fecha_vencimiento IS NOT NULL";
			}
			elseif ($this->FechaVencimiento === false) {
				$aWhere[] = "servicios.fecha_vencimiento IS NULL";
			}
			else {
				$aWhere[] = "servicios.fecha_vencimiento = '{$this->FechaVencimiento}'";
			}
		}
		if ($this->FechaFacturacion !== null) {
			$aWhere[] = "servicios.fecha_facturacion = '{$this->FechaFacturacion}'";
		}
		if ($this->IdPeriodo !== null) {
			$aWhere[] = "servicios.id_periodo = '{$this->IdPeriodo}'";
		}
		if ($this->Importe !== null) {
			$aWhere[] = "servicios.importe = '{$this->Importe}'";
		}
		if ($this->Descripcion !== null) {
			$aWhere[] = "servicios.descripcion = '{$this->Descripcion}'";
		}
		if ($this->Comentario !== null) {
			$aWhere[] = "servicios.comentario = '{$this->Comentario}'";
		}
		if ($this->IdOrden !== null) {
			$aWhere[] = "servicios.id_orden = '{$this->IdOrden}'";
		}
		if (@$this->PorVencer === true) {
			$aWhere[] = "servicios.fecha_baja IS NULL";
			$aWhere[] = "SUBDATE(servicios.fecha_vencimiento, INTERVAL 30 DAY) <= NOW()";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "
			SELECT 
				SQL_CALC_FOUND_ROWS servicios.*, clientes.nombre cliente, clientes.categoria categoria_cliente, tipos_servicio.nombre tipo_servicio, proyectos.nombre proyecto, periodos.nombre periodo
			FROM 
				servicios 
				LEFT JOIN clientes ON clientes.id_cliente = servicios.id_cliente
				LEFT JOIN tipos_servicio ON tipos_servicio.id_tiposervicio = servicios.id_tiposervicio
				LEFT JOIN proyectos ON proyectos.id_proyecto = servicios.id_proyecto
				LEFT JOIN periodos ON periodos.id_periodo = servicios.id_periodo
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "
			SELECT 
				servicios.*, clientes.nombre cliente, tipos_servicio.nombre tipo_servicio, proyectos.nombre proyecto, periodos.nombre periodo
			FROM 
				servicios 
				LEFT JOIN clientes ON clientes.id_cliente = servicios.id_cliente
				LEFT JOIN tipos_servicio ON tipos_servicio.id_tiposervicio = servicios.id_tiposervicio
				LEFT JOIN proyectos ON proyectos.id_proyecto = servicios.id_proyecto
				LEFT JOIN periodos ON periodos.id_periodo = servicios.id_periodo
			WHERE servicios.id_servicio = '{$this->IdServicio}' 
			LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdServicio) {
			$cSql = "UPDATE servicios SET ".
				"id_tiposervicio = '{$this->IdTiposervicio}', ".
				"horas = ".($this->Horas > 0 ? "'{$this->Horas}'" : 'NULL').", ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"nombre = '{$this->Nombre}', ".
				"fecha_alta = '{$this->FechaAlta}', ".
				"fecha_activacion = ".($this->FechaActivacion ? "'{$this->FechaActivacion}'" : 'NULL').", ".
				"fecha_baja = ".($this->FechaBaja ? "'{$this->FechaBaja}'" : 'NULL').", ".
				"fecha_facturacion = '{$this->FechaFacturacion}', ".
				"fecha_vencimiento = '{$this->FechaVencimiento}', ".
				"id_periodo = '{$this->IdPeriodo}', ".
				"importe = '{$this->Importe}', ".
				"cuotas = '{$this->Cuotas}', ".
				"texto_orden = ".($this->TextoOrden ? "'{$this->TextoOrden}'" : 'NULL').", ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL').", ".
				"id_orden = ".($this->IdOrden ? "'{$this->IdOrden}'" : 'NULL')." ".
				"WHERE id_servicio = '{$this->IdServicio}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO servicios SET ".
				"id_tiposervicio = '{$this->IdTiposervicio}', ".
				"horas = ".($this->Horas > 0 ? "'{$this->Horas}'" : 'NULL').", ".
				"id_cliente = '{$this->IdCliente}', ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"nombre = '{$this->Nombre}', ".
				"fecha_alta = '{$this->FechaAlta}', ".
				"fecha_activacion = ".($this->FechaActivacion ? "'{$this->FechaActivacion}'" : 'NULL').", ".
				"fecha_baja = ".($this->FechaBaja ? "'{$this->FechaBaja}'" : 'NULL').", ".
				"fecha_facturacion = '{$this->FechaFacturacion}', ".
				"fecha_vencimiento = '{$this->FechaVencimiento}', ".
				"id_periodo = '{$this->IdPeriodo}', ".
				"importe = '{$this->Importe}', ".
				"cuotas = '{$this->Cuotas}', ".
				"texto_orden = ".($this->TextoOrden ? "'{$this->TextoOrden}'" : 'NULL').", ".
				"descripcion = ".($this->Descripcion ? "'{$this->Descripcion}'" : 'NULL').", ".
				"comentario = ".($this->Comentario ? "'{$this->Comentario}'" : 'NULL').", ".
				"id_orden = ".($this->IdOrden ? "'{$this->IdOrden}'" : 'NULL')."";
			
			$this->IdServicio = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdServicio;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM servicios WHERE id_servicio = '{$this->IdServicio}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdServicio = $aDatos['id_servicio'];
		$this->IdTiposervicio = $aDatos['id_tiposervicio'];
		$this->Horas = $aDatos['horas'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->Nombre = $aDatos['nombre'];
		$this->FechaAlta = $aDatos['fecha_alta'];
		$this->FechaActivacion = $aDatos['fecha_activacion'];
		$this->FechaBaja = $aDatos['fecha_baja'];
		$this->FechaFacturacion = $aDatos['fecha_facturacion'];
		$this->FechaVencimiento = $aDatos['fecha_vencimiento'];
		$this->IdPeriodo = $aDatos['id_periodo'];
		$this->Importe = $aDatos['importe'];
		$this->Cuotas = $aDatos['cuotas'];
		$this->TextoOrden = $aDatos['texto_orden'];
		$this->Descripcion = $aDatos['descripcion'];
		$this->Comentario = $aDatos['comentario'];
		$this->IdOrden = $aDatos['id_orden'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdServicio = $this->IdTiposervicio = $this->Horas = $this->IdCliente = $this->IdProyecto = $this->Nombre = $this->FechaAlta = $this->FechaActivacion = $this->FechaBaja = $this->FechaFacturacion = $this->FechaVencimiento = $this->IdPeriodo = $this->Importe = $this->Cuotas = $this->TextoOrden = $this->Descripcion = $this->Comentario = $this->IdOrden = null;
	}
}
?>