<?
class Tareas extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdTarea, $IdSector, $IdEntidad, $IdOperador, $FechaCreada, $FechaCerrada, $Texto, $Cerrada;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdSector !== null) {
			$aWhere[] = "tareas.id_sector = '{$this->IdSector}'";
		}
		if ($this->IdEntidad !== null) {
			$aWhere[] = "tareas.id_entidad = '{$this->IdEntidad}'";
		}
		if ($this->IdOperador !== null) {
			$aWhere[] = "tareas.id_operador = '{$this->IdOperador}'";
		}
		if ($this->FechaCreada !== null) {
			$aWhere[] = "tareas.fecha_creada = '{$this->FechaCreada}'";
		}
		if ($this->FechaCerrada !== null) {
			$aWhere[] = "tareas.fecha_cerrada = '{$this->FechaCerrada}'";
		}
		if ($this->Texto !== null) {
			$aWhere[] = "tareas.texto = '{$this->Texto}'";
		}
		if ($this->Cerrada !== null) {
			$aWhere[] = "tareas.cerrada = '{$this->Cerrada}'";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "
			SELECT SQL_CALC_FOUND_ROWS 
				tareas.*, operadores.nombre operador
			FROM tareas 
				LEFT JOIN operadores ON operadores.id_operador = tareas.id_operador
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT tareas.* FROM tareas WHERE tareas.id_tarea = '{$this->IdTarea}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdTarea) {
			$cSql = "UPDATE tareas SET ".
				"id_sector = '{$this->IdSector}', ".
				"id_entidad = '{$this->IdEntidad}', ".
				"id_operador = '{$this->IdOperador}', ".
				"texto = '{$this->Texto}', ".
				"fecha_cerrada = ".($this->FechaCerrada ? "'{$this->FechaCerrada}'" : 'NULL').", ".
				"cerrada = '{$this->Cerrada}' ".
				"WHERE id_tarea = '{$this->IdTarea}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO tareas SET ".
				"id_sector = '{$this->IdSector}', ".
				"id_entidad = '{$this->IdEntidad}', ".
				"id_operador = '{$this->IdOperador}', ".
				"texto = '{$this->Texto}', ".
				"fecha_cerrada = ".($this->FechaCerrada ? "'{$this->FechaCerrada}'" : 'NULL').", ".
				"cerrada = '{$this->Cerrada}'";
			
			$this->IdTarea = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdTarea;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM tareas WHERE id_tarea = '{$this->IdTarea}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdTarea = $aDatos['id_tarea'];
		$this->IdSector = $aDatos['id_sector'];
		$this->IdEntidad = $aDatos['id_entidad'];
		$this->IdOperador = $aDatos['id_operador'];
		$this->FechaCreada = $aDatos['fecha_creada'];
		$this->FechaCerrada = $aDatos['fecha_cerrada'];
		$this->Texto = $aDatos['texto'];
		$this->Cerrada = $aDatos['cerrada'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdTarea = $this->IdSector = $this->IdEntidad = $this->IdOperador = $this->FechaCreada = $this->FechaCerrada = $this->Texto = $this->Cerrada = null;
	}
}
?>