<?
class Tickets extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdTicket, $FechaIngreso, $FechaResuelto, $IdCliente, $IdProyecto, $Mensaje, $Respuesta, $Avisar;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->FechaIngreso !== null) {
			$aWhere[] = "tickets.fecha_ingreso = '{$this->FechaIngreso}'";
		}
		if ($this->FechaResuelto !== null) {
			$aWhere[] = "tickets.fecha_resuelto = '{$this->FechaResuelto}'";
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "tickets.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "tickets.id_proyecto = '{$this->IdProyecto}'";
		}
		if ($this->Mensaje !== null) {
			$aWhere[] = "tickets.mensaje = '{$this->Mensaje}'";
		}
		if ($this->Respuesta !== null) {
			$aWhere[] = "tickets.respuesta = '{$this->Respuesta}'";
		}
		if (isset($this->Abierto)) {
			if ($this->Abierto === true) {
				$aWhere[] = "tickets.fecha_resuelto IS NULL";
			}
			elseif ($this->Abierto === false) {
				$aWhere[] = "tickets.fecha_resuelto IS NOT NULL";
			}
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS tickets.*, clientes.nombre cliente, proyectos.nombre proyecto
			FROM tickets 
			INNER JOIN elementos ON elementos.id_entidad = tickets.id_ticket AND elementos.id_tipoelemento = ".ELEMENTOS_TICKET." AND elementos.eliminado = 0 
			LEFT JOIN clientes ON clientes.id_cliente = tickets.id_cliente
			LEFT JOIN proyectos ON proyectos.id_proyecto = tickets.id_proyecto
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT tickets.*, clientes.nombre cliente, proyectos.nombre proyecto
			FROM tickets 
			LEFT JOIN clientes ON clientes.id_cliente = tickets.id_cliente
			LEFT JOIN proyectos ON proyectos.id_proyecto = tickets.id_proyecto
			WHERE tickets.id_ticket = '{$this->IdTicket}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdTicket) {
			$cSql = "UPDATE tickets SET ".
				"fecha_ingreso = '{$this->FechaIngreso}', ".
				"fecha_resuelto = ".($this->FechaResuelto ? "'{$this->FechaResuelto}'" : 'NULL').", ".
				"id_cliente = ".($this->IdCliente ? "'{$this->IdCliente}'" : 'NULL').", ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"mensaje = ".($this->Mensaje ? "'{$this->Mensaje}'" : 'NULL').", ".
				"avisar = ".($this->Avisar ? "'{$this->Avisar}'" : 'NULL').", ".
				"respuesta = ".($this->Respuesta ? "'{$this->Respuesta}'" : 'NULL')." ".
				"WHERE id_ticket = '{$this->IdTicket}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO tickets SET ".
				"fecha_ingreso = '{$this->FechaIngreso}', ".
				"fecha_resuelto = ".($this->FechaResuelto ? "'{$this->FechaResuelto}'" : 'NULL').", ".
				"id_cliente = ".($this->IdCliente ? "'{$this->IdCliente}'" : 'NULL').", ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"mensaje = ".($this->Mensaje ? "'{$this->Mensaje}'" : 'NULL').", ".
				"avisar = ".($this->Avisar ? "'{$this->Avisar}'" : 'NULL').", ".
				"respuesta = ".($this->Respuesta ? "'{$this->Respuesta}'" : 'NULL')."";
			
			$this->IdTicket = $this->DB->QueryInsert($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_TICKET, $this->IdTicket);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdTicket;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		Utils::EliminarElemento(ELEMENTOS_TICKET, $this->IdTicket);
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdTicket = $aDatos['id_ticket'];
		$this->FechaIngreso = $aDatos['fecha_ingreso'];
		$this->FechaResuelto = $aDatos['fecha_resuelto'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->Mensaje = $aDatos['mensaje'];
		$this->Respuesta = $aDatos['respuesta'];
		$this->Avisar = $aDatos['avisar'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdTicket = $this->FechaIngreso = $this->FechaResuelto = $this->IdCliente = $this->IdProyecto = $this->Mensaje = $this->Respuesta = $this->Avisar = null;
	}
}
?>