<?
class Tipos_egresos extends Model {
	
	/**
	 * @desc
	 * Caso especial para buscar por letra inicial
	 */
	public $PorLetra;
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdTipoegreso, $Nombre;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de Tipos_egresos.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Nombre !== null) {
			$aWhere[] = "tipos_egresos.nombre LIKE '%{$this->Nombre}%'";
		}
		
		switch ($this->PorLetra) {
			case 'numeros':
				$aWhere[] = "tipos_egresos.nombre REGEXP '^[^a-z]'";
				break;
				
			case null:
				break;
			
			default:
				$aWhere[] = "tipos_egresos.nombre LIKE BINARY '{$this->PorLetra}%'";
				break;
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS tipos_egresos.* FROM tipos_egresos INNER JOIN elementos ON elementos.id_entidad = tipos_egresos.id_tipoegreso AND elementos.id_tipoelemento = ".ELEMENTOS_TIPOEGRESO." AND elementos.eliminado = 0 {$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el Tipos_egresos con ID $this->IdTipoegreso.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT tipos_egresos.* FROM tipos_egresos WHERE tipos_egresos.id_tipoegreso = '{$this->IdTipoegreso}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un Tipos_egresos.
	 * Devuelve el ID del Tipos_egresos guardado.
	 *
	 * @param array $aData
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdTipoegreso) {
			$cSql = "UPDATE tipos_egresos SET ".
				"nombre = '{$this->Nombre}' ".
				"WHERE id_tipoegreso = '{$this->IdTipoegreso}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO tipos_egresos SET ".
				"nombre = '{$this->Nombre}'";
			
			$this->IdTipoegreso = $this->DB->QueryInsert($cSql);
			
			Utils::GuardarElemento(ELEMENTOS_TIPOEGRESO, $this->IdTipoegreso);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdTipoegreso;
		}
	}
	
	/**
	 * @desc
	 * Elimina un Tipos_egresos.
	 *
	 */
	public function Eliminar() {
		$this->DB->Begin();
		
		Utils::EliminarElemento(ELEMENTOS_TIPOEGRESO, $this->IdTipoegreso);
		
		$this->DB->Commit();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdTipoegreso = $aDatos['id_tipoegreso'];
		$this->Nombre = $aDatos['nombre'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdTipoegreso = $this->Nombre = null;
	}
}
?>