<?
class Vencimientos extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdVencimiento, $Nombre, $IdPeriodo, $Visto;
	
	public $PorVencer = false;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cOrderBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->Nombre !== null) {
			$aWhere[] = "vencimientos.nombre = '{$this->Nombre}'";
		}
		if ($this->IdPeriodo !== null) {
			$aWhere[] = "vencimientos.id_periodo = '{$this->IdPeriodo}'";
		}
		if ($this->Visto !== null) {
			$aWhere[] = "vencimientos.visto = '{$this->Visto}'";
		}
		if ($this->PorVencer) {
			$aWhere[] = "ADDDATE(vencimientos.visto, INTERVAL vencimientos.id_periodo MONTH) <= NOW()";
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "SELECT SQL_CALC_FOUND_ROWS vencimientos.*, periodos.nombre periodo, ADDDATE(vencimientos.visto, INTERVAL vencimientos.id_periodo MONTH) proximo
			FROM vencimientos 
			LEFT JOIN periodos USING(id_periodo)
			{$cWhere} {$cOrderBy} {$cLimit}";
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "SELECT vencimientos.*, periodos.nombre periodo
			FROM vencimientos 
			LEFT JOIN periodos USING(id_periodo)
			WHERE vencimientos.id_vencimiento = '{$this->IdVencimiento}' LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		$cVisto = "'{$this->Visto}'";
		
		if (@$this->Sumar) {
			$cVisto = "ADDDATE(visto, INTERVAL {$this->IdPeriodo} MONTH)";
		}
		
		if ($this->IdVencimiento) {
			$cSql = "UPDATE vencimientos SET ".
				"nombre = '{$this->Nombre}', ".
				"id_periodo = '{$this->IdPeriodo}', ".
				"visto = {$cVisto} ".
				"WHERE id_vencimiento = '{$this->IdVencimiento}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO vencimientos SET ".
				"nombre = '{$this->Nombre}', ".
				"id_periodo = '{$this->IdPeriodo}', ".
				"visto = {$cVisto}";
			
			$this->IdVencimiento = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdVencimiento;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM vencimientos WHERE id_vencimiento = '{$this->IdVencimiento}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdVencimiento = $aDatos['id_vencimiento'];
		$this->Nombre = $aDatos['nombre'];
		$this->IdPeriodo = $aDatos['id_periodo'];
		$this->Visto = $aDatos['visto'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdVencimiento = $this->Nombre = $this->IdPeriodo = $this->Visto = null;
	}
}
?>