<?
class Ventas extends Model {
	
	/**
	 * @desc
	 * Cadena con el ORDER BY.
	 *
	 * @var string
	 */
	public $OrderBy;
	
	/**
	 * @desc
	 * Numero de pagina.
	 * Comienza en 1 (1 = primer pagina).
	 * 
	 * Si se pasa un valor menor a 1 lo ignora.
	 *
	 * @var int
	 */
	public $Page;
	
	
	/**
	 * @desc
	 * Limite de consulta.
	 * Se usa junto a Page.
	 * 
	 * Si se asigna NULL no se hace un limite en la consulta.
	 *
	 * @var int
	 */
	public $LimitCant = APP_MYSQL_LIMIT;
	
	/**
	 * @desc
	 * Indica la cantidad total de registros si no se
	 * ubiera usado la clausula LIMIT en la consulta.
	 *
	 * @var int
	 */
	public $FoundRows;
	
	/**
	 * @desc
	 * Propiedades del modelo.
	 *
	 * @var mixed
	 */
	public $IdVenta, $IdComercial, $Fecha, $IdPrecliente, $IdCliente, $IdProyecto, $Importe, $Comision, $Notas;
	
	function __construct() {
		parent::__construct();
	}
	
	/**
	 * @desc
	 * Realiza una busqueda de registros.
	 * Ordena por la propiedad OrderBy.
	 * Limita registros por las propiedades Page y LimitCant.
	 * 
	 * Calcula la cantidad de registros sin el limite y lo pone en la
	 * propiedad FoundRows.
	 *
	 * @return array
	 */
	public function Buscar() {
		$cSelect = $cOrderBy = $cGroupBy = $cLimit = '';
		$aWhere = array();
		
		// Ordenacion.
		if ($this->OrderBy) {
			$cOrderBy = "ORDER BY {$this->OrderBy}";
		}
		
		// Paginacion.
		if ($this->Page < 1) {
			$this->Page = 1;
		}
		if ($this->LimitCant !== null) {
			$nLimit = ($this->Page - 1) * $this->LimitCant;
			$cLimit = "LIMIT {$nLimit}, {$this->LimitCant}";
		}
		else {
			$cLimit = '';
		}
		
		if ($this->IdComercial !== null) {
			$aWhere[] = "ventas.id_comercial = '{$this->IdComercial}'";
		}
		if ($this->Fecha !== null) {
			$aWhere[] = "ventas.fecha = '{$this->Fecha}'";
		}
		if (@$this->Desde !== null) {
			$aWhere[] = "ventas.fecha >= '{$this->Desde}'";
		}
		if (@$this->Hasta !== null) {
			$aWhere[] = "ventas.fecha <= '{$this->Hasta}'";
		}
		if ($this->IdPrecliente !== null) {
			$aWhere[] = "ventas.id_precliente = '{$this->IdPrecliente}'";
		}
		if ($this->IdCliente !== null) {
			$aWhere[] = "ventas.id_cliente = '{$this->IdCliente}'";
		}
		if ($this->IdProyecto !== null) {
			$aWhere[] = "ventas.id_proyecto = '{$this->IdProyecto}'";
		}
		if (@$this->AgruparComercial) {
			$cGroupBy = 'GROUP BY ventas.id_comercial';
			$cSelect = ', SUM(ventas.importe) importe';
		}
		
		// Arma la cadena SQL.
		$cWhere = $aWhere ? 'WHERE '.join(' AND ', $aWhere) : '';
		
		$cSql = "
			SELECT SQL_CALC_FOUND_ROWS ventas.*, operadores.nombre comercial, preclientes.razon_social precliente, clientes.nombre cliente {$cSelect}
			FROM ventas 
				LEFT JOIN operadores ON operadores.id_operador = ventas.id_comercial
				LEFT JOIN preclientes ON preclientes.id_precliente = ventas.id_precliente
				LEFT JOIN clientes ON clientes.id_cliente = ventas.id_cliente
			{$cWhere} {$cGroupBy} {$cOrderBy} {$cLimit}";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->GetRecordset();
		
		$cSql = "SELECT FOUND_ROWS() total";
		$this->DB->Query($cSql);
		
		$this->FoundRows = $this->DB->Field('total');
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Devuelve el registro segun su ID primario.
	 *
	 * @param int $nId
	 * @return array
	 */
	public function Obtener() {
		$cSql = "
			SELECT 
				ventas.*, operadores.nombre comercial, preclientes.razon_social precliente, clientes.nombre cliente, proyectos.nombre proyecto
			FROM ventas 
				LEFT JOIN operadores ON operadores.id_operador = ventas.id_comercial
				LEFT JOIN preclientes ON preclientes.id_precliente = ventas.id_precliente
				LEFT JOIN clientes ON clientes.id_cliente = ventas.id_cliente
				LEFT JOIN proyectos ON proyectos.id_proyecto = ventas.id_proyecto
			WHERE ventas.id_venta = '{$this->IdVenta}' 
			LIMIT 1";
		
		$this->DB->Query($cSql);
		$aRet = $this->DB->Field();
		
		if ($aRet) {
			$this->Set($aRet);
		}
		else {
			$this->Limpiar();
		}
		
		return $aRet;
	}
	
	/**
	 * @desc
	 * Guarda un registro y devuelve su ID.
	 *
	 * @return int
	 */
	public function Guardar() {
		$this->DB->Begin();
		
		if ($this->IdVenta) {
			$cSql = "UPDATE ventas SET ".
				"id_comercial = '{$this->IdComercial}', ".
				"fecha = '{$this->Fecha}', ".
				"id_precliente = ".($this->IdPrecliente ? "'{$this->IdPrecliente}'" : 'NULL').", ".
				"id_cliente = ".($this->IdCliente ? "'{$this->IdCliente}'" : 'NULL').", ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"importe = '{$this->Importe}', ".
				"comision = '{$this->Comision}', ".
				"notas = ".($this->Notas ? "'{$this->Notas}'" : 'NULL')." ".
				"WHERE id_venta = '{$this->IdVenta}' LIMIT 1";
			
			$this->DB->Query($cSql);
		}
		else {
			$cSql = "INSERT INTO ventas SET ".
				"id_comercial = '{$this->IdComercial}', ".
				"fecha = '{$this->Fecha}', ".
				"id_precliente = ".($this->IdPrecliente ? "'{$this->IdPrecliente}'" : 'NULL').", ".
				"id_cliente = ".($this->IdCliente ? "'{$this->IdCliente}'" : 'NULL').", ".
				"id_proyecto = ".($this->IdProyecto ? "'{$this->IdProyecto}'" : 'NULL').", ".
				"importe = '{$this->Importe}', ".
				"comision = '{$this->Comision}', ".
				"notas = ".($this->Notas ? "'{$this->Notas}'" : 'NULL')."";
			
			$this->IdVenta = $this->DB->QueryInsert($cSql);
		}
		
		if ($this->DB->GetLastError()) {
			$this->DB->Rollback();
			return 0;
		}
		else {
			$this->DB->Commit();
			return $this->IdVenta;
		}
	}
	
	/**
	 * @desc
	 * Elimina un registro.
	 *
	 */
	public function Eliminar() {
		$cSql = "DELETE IGNORE FROM ventas WHERE id_venta = '{$this->IdVenta}' LIMIT 1";
		$this->DB->Query($cSql);
		
		return $this->DB->AffectedRows();
	}
	
	/**
	 * @desc
	 * Llena las propiedades del modelo con los datos
	 * provistos en el array $aDatos.
	 *
	 * @param array $aDatos
	 */
	private function Set($aDatos) {
		$this->IdVenta = $aDatos['id_venta'];
		$this->IdComercial = $aDatos['id_comercial'];
		$this->Fecha = $aDatos['fecha'];
		$this->IdPrecliente = $aDatos['id_precliente'];
		$this->IdCliente = $aDatos['id_cliente'];
		$this->IdProyecto = $aDatos['id_proyecto'];
		$this->Importe = $aDatos['importe'];
		$this->Comision = $aDatos['comision'];
		$this->Notas = $aDatos['notas'];
	}
	
	/**
	 * @desc
	 * Limpia las propiedades del objeto.
	 *
	 */
	public function Limpiar() {
		$this->IdVenta = $this->IdComercial = $this->Fecha = $this->IdPrecliente = $this->IdCliente = $this->IdProyecto = $this->Importe = $this->Comision = $this->Notas = null;
	}
}
?>